# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
# - Alexis Sukrieh
package FDN::Common::CharTable;

##############################################################
# Dclaration du module
##############################################################
use strict;
use Exporter ;
our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
	$iso8859_15_to_rfc822
	$iso8859_15_to_unixlogin
	$iso8859_15_to_ebcdic
	$ebcdic_to_iso8859_15
	&cleanup_chars_as_table
);

use warnings;

our $iso8859_15_to_rfc822 = {
	"\x00" => '',  # NUL
	"\x01" => '',  # SOH
	"\x02" => '',  # STX
	"\x03" => '',  # ETX
	"\x04" => '',  # EOT
	"\x05" => '',  # ENQ
	"\x06" => '',  # ACK
	"\x07" => '',  # BEL
	"\x08" => '',  # BS
	"\x09" => ' ',  # HT
	"\x0a" => ' ',  # LF
	"\x0b" => '',  # VT
	"\x0c" => '',  # FF
	"\x0d" => '',  # CR
	"\x0e" => '',  # SO
	"\x0f" => '',  # SI
	"\x10" => '',  # DLE
	"\x11" => '',  # DC1
	"\x12" => '',  # DC2
	"\x13" => '',  # DC3
	"\x14" => '',  # DC4
	"\x15" => '',  # NAK
	"\x16" => '',  # SYN
	"\x17" => '',  # ETB
	"\x18" => '',  # CAN
	"\x19" => '',  # EM
	"\x1a" => '',  # SUB
	"\x1b" => '',  # ESC
	"\x1c" => '',  # FS
	"\x1d" => '',  # GS
	"\x1e" => '',  # RS
	"\x1f" => '',  # US
	"\x20" => ' ',  # Espace
	"\x21" => '',  # !
	"\x22" => '',  # "
	"\x23" => '',  # #
	"\x24" => '',  # $
	"\x25" => '',  # %
	"\x26" => '',  # &
	"\x27" => '',  # ' 
	"\x28" => '',  # (
	"\x29" => '',  # )
	"\x2a" => '',  # *
	"\x2b" => '',  # +
	"\x2c" => '',  # ,
	"\x2d" => '-',  # -
	"\x2e" => ' ',  # .
	"\x2f" => '',  # /
	"\x30" => '0',  # 0
	"\x31" => '1',  # 1
	"\x32" => '2',  # 2
	"\x33" => '3',  # 3
	"\x34" => '4',  # 4
	"\x35" => '5',  # 5
	"\x36" => '6',  # 6
	"\x37" => '7',  # 7
	"\x38" => '8',  # 8
	"\x39" => '9',  # 9
	"\x3a" => '',  # :
	"\x3b" => '',  # ;
	"\x3c" => '',  # <
	"\x3d" => '',  # =
	"\x3e" => '',  # >
	"\x3f" => '',  # ?
	"\x40" => '',  # @
	"\x41" => 'a',  # A
	"\x42" => 'b',  #
	"\x43" => 'c',  #
	"\x44" => 'd',  #
	"\x45" => 'e',  #
	"\x46" => 'f',  #
	"\x47" => 'g',  #
	"\x48" => 'h',  #
	"\x49" => 'i',  #
	"\x4a" => 'j',  #
	"\x4b" => 'k',  #
	"\x4c" => 'l',  #
	"\x4d" => 'm',  #
	"\x4e" => 'n',  #
	"\x4f" => 'o',  #
	"\x50" => 'p',  #
	"\x51" => 'q',  #
	"\x52" => 'r',  #
	"\x53" => 's',  #
	"\x54" => 't',  #
	"\x55" => 'u',  #
	"\x56" => 'v',  #
	"\x57" => 'w',  #
	"\x58" => 'x',  #
	"\x59" => 'y',  #
	"\x5a" => 'z',  # Z
	"\x5b" => '',  # [
	"\x5c" => '',  # \
	"\x5d" => '',  # ]
	"\x5e" => '',  # ^
	"\x5f" => '_',  # _
	"\x60" => '',  # `
	"\x61" => 'a',  # a
	"\x62" => 'b',  #
	"\x63" => 'c',  #
	"\x64" => 'd',  #
	"\x65" => 'e',  #
	"\x66" => 'f',  #
	"\x67" => 'g',  #
	"\x68" => 'h',  #
	"\x69" => 'i',  #
	"\x6a" => 'j',  #
	"\x6b" => 'k',  #
	"\x6c" => 'l',  #
	"\x6d" => 'm',  #
	"\x6e" => 'n',  #
	"\x6f" => 'o',  #
	"\x70" => 'p',  #
	"\x71" => 'q',  #
	"\x72" => 'r',  #
	"\x73" => 's',  #
	"\x74" => 't',  #
	"\x75" => 'u',  #
	"\x76" => 'v',  #
	"\x77" => 'w',  #
	"\x78" => 'x',  #
	"\x79" => 'y',  #
	"\x7a" => 'z',  # z
	"\x7b" => '',  # {
	"\x7c" => '',  # |
	"\x7d" => '',  # }
	"\x7e" => '',  # ~
	"\x7f" => '',  # DEL
	"\x80" => '',  #
	"\x81" => '',  #
	"\x82" => '',  #
	"\x83" => '',  #
	"\x84" => '',  #
	"\x85" => '',  #
	"\x86" => '',  #
	"\x87" => '',  #
	"\x88" => '',  #
	"\x89" => '',  #
	"\x8a" => '',  #
	"\x8b" => '',  #
	"\x8c" => '',  #
	"\x8d" => '',  #
	"\x8e" => '',  #
	"\x8f" => '',  #
	"\x90" => '',  #
	"\x91" => '',  #
	"\x92" => '',  #
	"\x93" => '',  #
	"\x94" => '',  #
	"\x95" => '',  #
	"\x96" => '',  #
	"\x97" => '',  #
	"\x98" => '',  #
	"\x99" => '',  #
	"\x9a" => '',  #
	"\x9b" => '',  #
	"\x9c" => '',  #
	"\x9d" => '',  #
	"\x9e" => '',  #
	"\x9f" => '',  #
	"\xa0" => ' ',  # Espace ?
	"\xa1" => '',  # ! a l'envers
	"\xa2" => '',  # cent
	"\xa3" => '',  # livre sterling
	"\xa4" => '',  # euro
	"\xa5" => '',  # yen
	"\xa6" => '',  # S caron
	"\xa7" => '',  # paragraph
	"\xa8" => '',  # s caron
	"\xa9" => '',  # copyright
	"\xaa" => '',  # puissance a
	"\xab" => '',  # guillement francais ouvrant
	"\xac" => '',  # non logique
	"\xad" => '-',  # tiret ?
	"\xae" => '',  # registered
	"\xaf" => '',  # upperscroe :-)
	"\xb0" => '',  # degre
	"\xb1" => '',  # plus-moins
	"\xb2" => '',  # puissance deux
	"\xb3" => '',  # puissance trois
	"\xb4" => '',  # Z caron
	"\xb5" => '',  # mu
	"\xb6" => '',  # paragraphe americain (mouche)
	"\xb7" => ' ',  # point centre
	"\xb8" => '',  # z caron
	"\xb9" => '',  # puissance un
	"\xba" => '',  # degre souligne
	"\xbb" => '',  # guillemet fermant francais
	"\xbc" => 'oe',  # e dans l'o majuscule
	"\xbd" => 'oe',  # e dans l'o minuscule
	"\xbe" => 'y',  # Y trema majuscule
	"\xbf" => '',  # point d'interrogation a l'envers
	"\xc0" => 'a',  # 
	"\xc1" => 'a',  # 
	"\xc2" => 'a',  # 
	"\xc3" => 'a',  # ~A
	"\xc4" => 'a',  # 
	"\xc5" => 'a',  # \o{A}
	"\xc6" => 'ae',  # AE
	"\xc7" => 'c',  # 
	"\xc8" => 'e',  # 
	"\xc9" => 'e',  # 
	"\xca" => 'e',  # 
	"\xcb" => 'e',  # 
	"\xcc" => 'i',  # 
	"\xcd" => 'i',  # 
	"\xce" => 'i',  # 
	"\xcf" => 'i',  # 
	"\xd0" => 'd',  # Dahlgren
	"\xd1" => 'n',  # 
	"\xd2" => 'o',  # 
	"\xd3" => 'o',  # 
	"\xd4" => 'o',  # 
	"\xd5" => 'o',  # ~O
	"\xd6" => 'o',  # 
	"\xd7" => 'x',  # produit
	"\xd8" => 'o',  # O barre
	"\xd9" => 'u',  # 
	"\xda" => 'u',  # 
	"\xdb" => 'u',  # 
	"\xdc" => 'u',  # 
	"\xdd" => 'y',  # \'Y
	"\xde" => '',  # 
	"\xdf" => 'ss',  # esset
	"\xe0" => 'a',  # 
	"\xe1" => 'a',  # 
	"\xe2" => 'a',  # 
	"\xe3" => 'a',  # ~a
	"\xe4" => 'a',  # 
	"\xe5" => 'a',  # \o{a}
	"\xe6" => 'ae',  # e dans l'a
	"\xe7" => 'c',  # 
	"\xe8" => 'e',  # 
	"\xe9" => 'e',  # 
	"\xea" => 'e',  # 
	"\xeb" => 'e',  # 
	"\xec" => 'i',  # 
	"\xed" => 'i',  # 
	"\xee" => 'i',  # 
	"\xef" => 'i',  # 
	"\xf0" => 'd',  # d barre minuscule
	"\xf1" => 'n',  # 
	"\xf2" => 'o',  # 
	"\xf3" => 'o',  # 
	"\xf4" => 'o',  # 
	"\xf5" => 'o',  # ~o
	"\xf6" => 'o',  # 
	"\xf7" => '',  # divise
	"\xf8" => 'o',  # o barre
	"\xf9" => 'u',  # 
	"\xfa" => 'u',  # 
	"\xfb" => 'u',  # 
	"\xfc" => 'u',  # 
	"\xfd" => 'y',  # \'y
	"\xfe" => '',  # 
	"\xff" => 'y',  # \"y
};

our $iso8859_15_to_unixlogin = {
	"\x00" => '',  # NUL
	"\x01" => '',  # SOH
	"\x02" => '',  # STX
	"\x03" => '',  # ETX
	"\x04" => '',  # EOT
	"\x05" => '',  # ENQ
	"\x06" => '',  # ACK
	"\x07" => '',  # BEL
	"\x08" => '',  # BS
	"\x09" => '',  # HT
	"\x0a" => '',  # LF
	"\x0b" => '',  # VT
	"\x0c" => '',  # FF
	"\x0d" => '',  # CR
	"\x0e" => '',  # SO
	"\x0f" => '',  # SI
	"\x10" => '',  # DLE
	"\x11" => '',  # DC1
	"\x12" => '',  # DC2
	"\x13" => '',  # DC3
	"\x14" => '',  # DC4
	"\x15" => '',  # NAK
	"\x16" => '',  # SYN
	"\x17" => '',  # ETB
	"\x18" => '',  # CAN
	"\x19" => '',  # EM
	"\x1a" => '',  # SUB
	"\x1b" => '',  # ESC
	"\x1c" => '',  # FS
	"\x1d" => '',  # GS
	"\x1e" => '',  # RS
	"\x1f" => '',  # US
	"\x20" => '',  # Espace
	"\x21" => '',  # !
	"\x22" => '',  # "
	"\x23" => '',  # #
	"\x24" => '',  # $
	"\x25" => '',  # %
	"\x26" => '',  # &
	"\x27" => '',  # ' 
	"\x28" => '',  # (
	"\x29" => '',  # )
	"\x2a" => '',  # *
	"\x2b" => '',  # +
	"\x2c" => '',  # ,
	"\x2d" => '',  # -
	"\x2e" => '',  # .
	"\x2f" => '',  # /
	"\x30" => '0',  # 0
	"\x31" => '1',  # 1
	"\x32" => '2',  # 2
	"\x33" => '3',  # 3
	"\x34" => '4',  # 4
	"\x35" => '5',  # 5
	"\x36" => '6',  # 6
	"\x37" => '7',  # 7
	"\x38" => '8',  # 8
	"\x39" => '9',  # 9
	"\x3a" => '',  # :
	"\x3b" => '',  # ;
	"\x3c" => '',  # <
	"\x3d" => '',  # =
	"\x3e" => '',  # >
	"\x3f" => '',  # ?
	"\x40" => '',  # @
	"\x41" => 'a',  # A
	"\x42" => 'b',  #
	"\x43" => 'c',  #
	"\x44" => 'd',  #
	"\x45" => 'e',  #
	"\x46" => 'f',  #
	"\x47" => 'g',  #
	"\x48" => 'h',  #
	"\x49" => 'i',  #
	"\x4a" => 'j',  #
	"\x4b" => 'k',  #
	"\x4c" => 'l',  #
	"\x4d" => 'm',  #
	"\x4e" => 'n',  #
	"\x4f" => 'o',  #
	"\x50" => 'p',  #
	"\x51" => 'q',  #
	"\x52" => 'r',  #
	"\x53" => 's',  #
	"\x54" => 't',  #
	"\x55" => 'u',  #
	"\x56" => 'v',  #
	"\x57" => 'w',  #
	"\x58" => 'x',  #
	"\x59" => 'y',  #
	"\x5a" => 'z',  # Z
	"\x5b" => '',  # [
	"\x5c" => '',  # \
	"\x5d" => '',  # ]
	"\x5e" => '',  # ^
	"\x5f" => '',  # _
	"\x60" => '',  # `
	"\x61" => 'a',  # a
	"\x62" => 'b',  #
	"\x63" => 'c',  #
	"\x64" => 'd',  #
	"\x65" => 'e',  #
	"\x66" => 'f',  #
	"\x67" => 'g',  #
	"\x68" => 'h',  #
	"\x69" => 'i',  #
	"\x6a" => 'j',  #
	"\x6b" => 'k',  #
	"\x6c" => 'l',  #
	"\x6d" => 'm',  #
	"\x6e" => 'n',  #
	"\x6f" => 'o',  #
	"\x70" => 'p',  #
	"\x71" => 'q',  #
	"\x72" => 'r',  #
	"\x73" => 's',  #
	"\x74" => 't',  #
	"\x75" => 'u',  #
	"\x76" => 'v',  #
	"\x77" => 'w',  #
	"\x78" => 'x',  #
	"\x79" => 'y',  #
	"\x7a" => 'z',  # z
	"\x7b" => '',  # {
	"\x7c" => '',  # |
	"\x7d" => '',  # }
	"\x7e" => '',  # ~
	"\x7f" => '',  # DEL
	"\x80" => '',  #
	"\x81" => '',  #
	"\x82" => '',  #
	"\x83" => '',  #
	"\x84" => '',  #
	"\x85" => '',  #
	"\x86" => '',  #
	"\x87" => '',  #
	"\x88" => '',  #
	"\x89" => '',  #
	"\x8a" => '',  #
	"\x8b" => '',  #
	"\x8c" => '',  #
	"\x8d" => '',  #
	"\x8e" => '',  #
	"\x8f" => '',  #
	"\x90" => '',  #
	"\x91" => '',  #
	"\x92" => '',  #
	"\x93" => '',  #
	"\x94" => '',  #
	"\x95" => '',  #
	"\x96" => '',  #
	"\x97" => '',  #
	"\x98" => '',  #
	"\x99" => '',  #
	"\x9a" => '',  #
	"\x9b" => '',  #
	"\x9c" => '',  #
	"\x9d" => '',  #
	"\x9e" => '',  #
	"\x9f" => '',  #
	"\xa0" => '',  # Espace ?
	"\xa1" => '',  # ! a l'envers
	"\xa2" => '',  # cent
	"\xa3" => '',  # livre sterling
	"\xa4" => '',  # euro
	"\xa5" => '',  # yen
	"\xa6" => '',  # S caron
	"\xa7" => '',  # paragraph
	"\xa8" => '',  # s caron
	"\xa9" => '',  # copyright
	"\xaa" => '',  # puissance a
	"\xab" => '',  # guillement francais ouvrant
	"\xac" => '',  # non logique
	"\xad" => '',  # tiret ?
	"\xae" => '',  # registered
	"\xaf" => '',  # upperscroe :-)
	"\xb0" => '',  # degre
	"\xb1" => '',  # plus-moins
	"\xb2" => '',  # puissance deux
	"\xb3" => '',  # puissance trois
	"\xb4" => '',  # Z caron
	"\xb5" => '',  # mu
	"\xb6" => '',  # paragraphe americain (mouche)
	"\xb7" => '',  # point centre
	"\xb8" => '',  # z caron
	"\xb9" => '',  # puissance un
	"\xba" => '',  # degre souligne
	"\xbb" => '',  # guillemet fermant francais
	"\xbc" => 'oe',  # e dans l'o majuscule
	"\xbd" => 'oe',  # e dans l'o minuscule
	"\xbe" => 'y',  # Y trema majuscule
	"\xbf" => '',  # point d'interrogation a l'envers
	"\xc0" => 'a',  # 
	"\xc1" => 'a',  # 
	"\xc2" => 'a',  # 
	"\xc3" => 'a',  # ~A
	"\xc4" => 'a',  # 
	"\xc5" => 'a',  # \o{A}
	"\xc6" => 'ae',  # AE
	"\xc7" => 'c',  # 
	"\xc8" => 'e',  # 
	"\xc9" => 'e',  # 
	"\xca" => 'e',  # 
	"\xcb" => 'e',  # 
	"\xcc" => 'i',  # 
	"\xcd" => 'i',  # 
	"\xce" => 'i',  # 
	"\xcf" => 'i',  # 
	"\xd0" => 'd',  # Dahlgren
	"\xd1" => 'n',  # 
	"\xd2" => 'o',  # 
	"\xd3" => 'o',  # 
	"\xd4" => 'o',  # 
	"\xd5" => 'o',  # ~O
	"\xd6" => 'o',  # 
	"\xd7" => 'x',  # produit
	"\xd8" => 'o',  # O barre
	"\xd9" => 'u',  # 
	"\xda" => 'u',  # 
	"\xdb" => 'u',  # 
	"\xdc" => 'u',  # 
	"\xdd" => 'y',  # \'Y
	"\xde" => '',  # 
	"\xdf" => 'ss',  # esset
	"\xe0" => 'a',  # 
	"\xe1" => 'a',  # 
	"\xe2" => 'a',  # 
	"\xe3" => 'a',  # ~a
	"\xe4" => 'a',  # 
	"\xe5" => 'a',  # \o{a}
	"\xe6" => 'ae',  # e dans l'a
	"\xe7" => 'c',  # 
	"\xe8" => 'e',  # 
	"\xe9" => 'e',  # 
	"\xea" => 'e',  # 
	"\xeb" => 'e',  # 
	"\xec" => 'i',  # 
	"\xed" => 'i',  # 
	"\xee" => 'i',  # 
	"\xef" => 'i',  # 
	"\xf0" => 'd',  # d barre minuscule
	"\xf1" => 'n',  # 
	"\xf2" => 'o',  # 
	"\xf3" => 'o',  # 
	"\xf4" => 'o',  # 
	"\xf5" => 'o',  # ~o
	"\xf6" => 'o',  # 
	"\xf7" => '',  # divise
	"\xf8" => 'o',  # o barre
	"\xf9" => 'u',  # 
	"\xfa" => 'u',  # 
	"\xfb" => 'u',  # 
	"\xfc" => 'u',  # 
	"\xfd" => 'y',  # \'y
	"\xfe" => '',  # 
	"\xff" => 'y',  # \"y
};

our $iso8859_15_to_ebcdic = {
	"\x00" => "\x00",  # NUL
	"\x01" => "\x01",  # SOH
	"\x02" => "\x02",  # STX
	"\x03" => "\x03",  # ETX
	"\x04" => "\x37",  # EOT
	"\x05" => "\x2d",  # ENQ
	"\x06" => "\x2e",  # ACK
	"\x07" => "\x2f",  # BEL
	"\x08" => "\x16",  # BS
	"\x09" => "\x05",  # HT
	"\x0a" => "\x25",  # LF
	"\x0b" => "\x0b",  # VT
	"\x0c" => "\x0c",  # FF
	"\x0d" => "\x0d",  # CR
	"\x0e" => "\x0e",  # SO
	"\x0f" => "\x0f",  # SI
	"\x10" => "\x10",  # DLE
	"\x11" => "\x11",  # DC1
	"\x12" => "\x12",  # DC2
	"\x13" => "",  # DC3
	"\x14" => "",  # DC4
	"\x15" => "\x3d",  # NAK
	"\x16" => "\x32",  # SYN
	"\x17" => "\x26",  # ETB
	"\x18" => "\x18",  # CAN
	"\x19" => "\x19",  # EM
	"\x1a" => "\x3f",  # SUB
	"\x1b" => "\x27",  # ESC
	"\x1c" => "\x1c",  # FS
	"\x1d" => "\x1d",  # GS
	"\x1e" => "\x1e",  # RS
	"\x1f" => "\x1f",  # US
	"\x20" => "\x40",  # Espace
	"\x21" => "\x5a",  # !
	"\x22" => "\x7f",  # "
	"\x23" => "\x7b",  # #
	"\x24" => "\x5b",  # $
	"\x25" => "\x6c",  # %
	"\x26" => "\x50",  # &
	"\x27" => "\x7d",  # ' 
	"\x28" => "\x4d",  # (
	"\x29" => "\x5d",  # )
	"\x2a" => "\x5c",  # *
	"\x2b" => "\x4e",  # +
	"\x2c" => "\x6b",  # ,
	"\x2d" => "\x60",  # -
	"\x2e" => "\x4b",  # .
	"\x2f" => "\x61",  # /
	"\x30" => "\xf0",  # 0
	"\x31" => "\xf1",  # 1
	"\x32" => "\xf2",  # 2
	"\x33" => "\xf3",  # 3
	"\x34" => "\xf4",  # 4
	"\x35" => "\xf5",  # 5
	"\x36" => "\xf6",  # 6
	"\x37" => "\xf7",  # 7
	"\x38" => "\xf8",  # 8
	"\x39" => "\xf9",  # 9
	"\x3a" => "\x7a",  # :
	"\x3b" => "\x5e",  # ;
	"\x3c" => "\x4c",  # <
	"\x3d" => "\x7e",  # =
	"\x3e" => "\x6e",  # >
	"\x3f" => "\x6f",  # ?
	"\x40" => "\x7c",  # @
	"\x41" => "\xc1",  # A
	"\x42" => "\xc2",  # B
	"\x43" => "\xc3",  # C
	"\x44" => "\xc4",  # D
	"\x45" => "\xc5",  # E
	"\x46" => "\xc6",  # F
	"\x47" => "\xc7",  # G
	"\x48" => "\xc8",  # H
	"\x49" => "\xc9",  # I
	"\x4a" => "\xd1",  # J
	"\x4b" => "\xd2",  # K
	"\x4c" => "\xd3",  # L
	"\x4d" => "\xd4",  # M
	"\x4e" => "\xd5",  # N
	"\x4f" => "\xd6",  # O
	"\x50" => "\xd7",  # P
	"\x51" => "\xd8",  # Q
	"\x52" => "\xd9",  # R
	"\x53" => "\xe2",  # S
	"\x54" => "\xe3",  # T
	"\x55" => "\xe4",  # U
	"\x56" => "\xe5",  # V
	"\x57" => "\xe6",  # W
	"\x58" => "\xe7",  # X
	"\x59" => "\xe8",  # Y
	"\x5a" => "\xe9",  # Z
	"\x5b" => "",  # [
	"\x5c" => "",  # \
	"\x5d" => "",  # ]
	"\x5e" => "",  # ^
	"\x5f" => "\x6d",  # _
	"\x60" => "",  # `
	"\x61" => "\x81",  # a
	"\x62" => "\x82",  # b
	"\x63" => "\x83",  # c
	"\x64" => "\x84",  # d
	"\x65" => "\x85",  # e
	"\x66" => "\x86",  # f
	"\x67" => "\x87",  # g
	"\x68" => "\x88",  # h
	"\x69" => "\x89",  # i
	"\x6a" => "\x91",  # j
	"\x6b" => "\x92",  # k
	"\x6c" => "\x93",  # l
	"\x6d" => "\x94",  # m
	"\x6e" => "\x95",  # n
	"\x6f" => "\x96",  # o
	"\x70" => "\x97",  # p
	"\x71" => "\x98",  # q
	"\x72" => "\x99",  # r
	"\x73" => "\xa2",  # s
	"\x74" => "\xa3",  # t
	"\x75" => "\xa4",  # u
	"\x76" => "\xa5",  # v
	"\x77" => "\xa6",  # w
	"\x78" => "\xa7",  # x
	"\x79" => "\xa8",  # y
	"\x7a" => "\xa9",  # z
	"\x7b" => "",  # {
	"\x7c" => "",  # |
	"\x7d" => "",  # }
	"\x7e" => "",  # ~
	"\x7f" => "\x07",  # DEL
	"\x80" => "",  #
	"\x81" => "",  #
	"\x82" => "",  #
	"\x83" => "",  #
	"\x84" => "",  #
	"\x85" => "",  #
	"\x86" => "",  #
	"\x87" => "",  #
	"\x88" => "",  #
	"\x89" => "",  #
	"\x8a" => "",  #
	"\x8b" => "",  #
	"\x8c" => "",  #
	"\x8d" => "",  #
	"\x8e" => "",  #
	"\x8f" => "",  #
	"\x90" => "",  #
	"\x91" => "",  #
	"\x92" => "",  #
	"\x93" => "",  #
	"\x94" => "",  #
	"\x95" => "",  #
	"\x96" => "",  #
	"\x97" => "",  #
	"\x98" => "",  #
	"\x99" => "",  #
	"\x9a" => "",  #
	"\x9b" => "",  #
	"\x9c" => "",  #
	"\x9d" => "",  #
	"\x9e" => "",  #
	"\x9f" => "",  #
	"\xa0" => "",  # Espace ?
	"\xa1" => "",  # ! a l'envers
	"\xa2" => "\x4a",  # cent
	"\xa3" => "\xd3",  # livre sterling -> L
	"\xa4" => "\xc5",  # euro -> E
	"\xa5" => "\xe8",  # yen -> Y
	"\xa6" => "\xe2",  # S caron -> S
	"\xa7" => "\xe2",  # paragraph -> S
	"\xa8" => "\xe2",  # s caron -> S
	"\xa9" => "\x4d\x83\x5d",  # copyright -> (c)
	"\xaa" => "\x81",  # puissance a -> a
	"\xab" => "\x4c\x4c",  # guillement francais ouvrant -> <<
	"\xac" => "\x5f",  # non logique
	"\xad" => "\x60",  # tiret ?
	"\xae" => "\x4d\x99\x5d",  # registered -> (r)
	"\xaf" => "",  # upperscroe :-)
	"\xb0" => "",  # degre
	"\xb1" => "",  # plus-moins
	"\xb2" => "\xf2",  # puissance deux -> 2
	"\xb3" => "\xf3",  # puissance trois -> 3
	"\xb4" => "\xe9",  # Z caron -> Z
	"\xb5" => "\xa4",  # mu -> u
	"\xb6" => "\xd7",  # paragraphe americain (mouche) -> P
	"\xb7" => "\x4b",  # point centre -> .
	"\xb8" => "\xa9",  # z caron -> z
	"\xb9" => "\xf1",  # puissance un -> 1
	"\xba" => "",  # degre souligne
	"\xbb" => "\x6e\x6e",  # guillemet fermant francais -> >>
	"\xbc" => "\xd6\xc5",  # e dans l'o majuscule -> OE
	"\xbd" => "\x96\x85",  # e dans l'o minuscule
	"\xbe" => "\xe8",  # Y trema majuscule
	"\xbf" => "",  # point d'interrogation a l'envers
	"\xc0" => "\xc1",  # 
	"\xc1" => "\xc1",  # 
	"\xc2" => "\xc1",  # 
	"\xc3" => "\xc1",  # ~A
	"\xc4" => "\xc1",  # 
	"\xc5" => "\xc1",  # \o{A}
	"\xc6" => "\xc1\xc5",  # AE
	"\xc7" => "\xc3",  # 
	"\xc8" => "\xc5",  # 
	"\xc9" => "\xc5",  # 
	"\xca" => "\xc5",  # 
	"\xcb" => "\xc5",  # 
	"\xcc" => "\xc9",  # 
	"\xcd" => "\xc9",  # 
	"\xce" => "\xc9",  # 
	"\xcf" => "\xc9",  # 
	"\xd0" => "\xc4",  # Dahlgren
	"\xd1" => "\xd5",  # 
	"\xd2" => "\xd6",  # 
	"\xd3" => "\xd6",  # 
	"\xd4" => "\xd6",  # 
	"\xd5" => "\xd6",  # ~O
	"\xd6" => "\xd6",  # 
	"\xd7" => "\xa7",  # produit
	"\xd8" => "\xd6",  # O barre
	"\xd9" => "\xe4",  # 
	"\xda" => "\xe4",  # 
	"\xdb" => "\xe4",  # 
	"\xdc" => "\xe4",  # 
	"\xdd" => "\xe8",  # \'Y
	"\xde" => "",  # 
	"\xdf" => "\xa2\xa2",  # esset
	"\xe0" => "\x81",  # 
	"\xe1" => "\x81",  # 
	"\xe2" => "\x81",  # 
	"\xe3" => "\x81",  # ~a
	"\xe4" => "\x81",  # 
	"\xe5" => "\x81",  # \o{a}
	"\xe6" => "\x81\x85",  # e dans l'a
	"\xe7" => "\x83",  # 
	"\xe8" => "\x85",  # 
	"\xe9" => "\x85",  # 
	"\xea" => "\x85",  # 
	"\xeb" => "\x85",  # 
	"\xec" => "\x89",  # 
	"\xed" => "\x89",  # 
	"\xee" => "\x89",  # 
	"\xef" => "\x89",  # 
	"\xf0" => "\x84",  # d barre minuscule
	"\xf1" => "\x95",  # 
	"\xf2" => "\x96",  # 
	"\xf3" => "\x96",  # 
	"\xf4" => "\x96",  # 
	"\xf5" => "\x96",  # ~o
	"\xf6" => "\x96",  # 
	"\xf7" => "\x61",  # divise -> /
	"\xf8" => "\x96",  # o barre
	"\xf9" => "\xa4",  # 
	"\xfa" => "\xa4",  # 
	"\xfb" => "\xa4",  # 
	"\xfc" => "\xa4",  # 
	"\xfd" => "\xa8",  # \'y
	"\xfe" => "",  # 
	"\xff" => "\xa8",  # \"y
};

our $ebcdic_to_iso8859_15 = {
	"\x00" => "\x00",  # NUL
	"\x01" => "\x01",  # SOH
	"\x02" => "\x02",  # STX
	"\x03" => "\x03",  # ETX
	"\x04" => "",  # PF
	"\x05" => "\x09",  # HT
	"\x06" => "",  # LC
	"\x07" => "\x7f",  # DEL
	"\x08" => "",  # GE
	"\x09" => "",  # RLF
	"\x0a" => "",  # SMM
	"\x0b" => "\x0b",  # VT
	"\x0c" => "\x0c",  # FF
	"\x0d" => "\x0d",  # CR
	"\x0e" => "\x0e",  # SO
	"\x0f" => "\x0f",  # SI
	"\x10" => "\x10",  # DLE
	"\x11" => "\x11",  # DC1
	"\x12" => "\x12",  # DC2
	"\x13" => "",  # TM
	"\x14" => "",  # RES
	"\x15" => "",  # NL
	"\x16" => "\x08",  # BS
	"\x17" => "",  # IL
	"\x18" => "\x18",  # CAN
	"\x19" => "\x19",  # EM
	"\x1a" => "",  # CC
	"\x1b" => "",  # CUI
	"\x1c" => "\x1c",  # IFS
	"\x1d" => "\x1d",  # IGS
	"\x1e" => "\x1e",  # IRS
	"\x1f" => "\x1f",  # IUS
	"\x20" => "",  # DS
	"\x21" => "",  # SOS
	"\x22" => "",  # FS
	"\x23" => "",  # 
	"\x24" => "",  # BYP
	"\x25" => "\x0a",  # LF
	"\x26" => "\x17",  # ETB
	"\x27" => "\x1b",  # ESC
	"\x28" => "",  #
	"\x29" => "",  #
	"\x2a" => "",  # SM
	"\x2b" => "",  # CU2
	"\x2c" => "",  # 
	"\x2d" => "\x05",  # ENQ
	"\x2e" => "\x06",  # ACK
	"\x2f" => "\x07",  # BEL
	"\x30" => "",  #
	"\x31" => "",  #
	"\x32" => "\x16",  # SYN
	"\x33" => "",  # 
	"\x34" => "",  # PN
	"\x35" => "",  # RS
	"\x36" => "",  # UC
	"\x37" => "\x04",  # EOT
	"\x38" => "",  # 
	"\x39" => "",  # 
	"\x3a" => "",  # 
	"\x3b" => "",  # CU3
	"\x3c" => "",  # DC4
	"\x3d" => "\x15",  # NAK
	"\x3e" => "",  # 
	"\x3f" => "\x1a",  # SUB
	"\x40" => "\x20",  # SP (Space?)
	"\x41" => "",  # 
	"\x42" => "",  #
	"\x43" => "",  #
	"\x44" => "",  #
	"\x45" => "",  #
	"\x46" => "",  #
	"\x47" => "",  #
	"\x48" => "",  #
	"\x49" => "",  #
	"\x4a" => "\xa2",  # cent
	"\x4b" => "\x2e",  # .
	"\x4c" => "\x3c",  # <
	"\x4d" => "\x28",  # (
	"\x4e" => "\x2b",  # +
	"\x4f" => "\x7c",  # |
	"\x50" => "\x26",  # &
	"\x51" => "",  #
	"\x52" => "",  #
	"\x53" => "",  #
	"\x54" => "",  #
	"\x55" => "",  #
	"\x56" => "",  #
	"\x57" => "",  #
	"\x58" => "",  #
	"\x59" => "",  #
	"\x5a" => "\x21",  # ! 
	"\x5b" => "\x24",  # $
	"\x5c" => "\x2a",  # *
	"\x5d" => "\x29",  # )
	"\x5e" => "\x3b",  # ;
	"\x5f" => "\xac",  # not logic
	"\x60" => "\x2d",  # -
	"\x61" => "\x2f",  # /
	"\x62" => "",  #
	"\x63" => "",  #
	"\x64" => "",  #
	"\x65" => "",  #
	"\x66" => "",  #
	"\x67" => "",  #
	"\x68" => "",  #
	"\x69" => "",  #
	"\x6a" => "",  # | with a blank in the middle
	"\x6b" => "\x2c",  # ,
	"\x6c" => "\x25",  # %
	"\x6d" => "\x5f",  # _
	"\x6e" => "\x3e",  # >
	"\x6f" => "\x3f",  # ?
	"\x70" => "",  #
	"\x71" => "",  #
	"\x72" => "",  #
	"\x73" => "",  #
	"\x74" => "",  #
	"\x75" => "",  #
	"\x76" => "",  #
	"\x77" => "",  #
	"\x78" => "",  #
	"\x79" => "\x60",  # `
	"\x7a" => "\x3a",  # :
	"\x7b" => "\x23",  # #
	"\x7c" => "\x40",  # @
	"\x7d" => "\x27",  # '
	"\x7e" => "\x3d",  # =
	"\x7f" => "\x22",  # "
	"\x80" => "",  # 
	"\x81" => "\x61",  # a
	"\x82" => "\x62",  # b
	"\x83" => "\x63",  # c
	"\x84" => "\x64",  # d
	"\x85" => "\x65",  # e
	"\x86" => "\x66",  # f
	"\x87" => "\x67",  # g
	"\x88" => "\x68",  # h
	"\x89" => "\x69",  # i
	"\x8a" => "",  #
	"\x8b" => "",  #
	"\x8c" => "",  #
	"\x8d" => "",  #
	"\x8e" => "",  #
	"\x8f" => "",  #
	"\x90" => "",  #
	"\x91" => "\x6a",  # j
	"\x92" => "\x6b",  # k
	"\x93" => "\x6c",  # l
	"\x94" => "\x6d",  # m
	"\x95" => "\x6e",  # n
	"\x96" => "\x6f",  # o
	"\x97" => "\x70",  # p
	"\x98" => "\x71",  # q
	"\x99" => "\x72",  # r
	"\x9a" => "",  #
	"\x9b" => "",  #
	"\x9c" => "",  #
	"\x9d" => "",  #
	"\x9e" => "",  #
	"\x9f" => "",  #
	"\xa0" => "",  # 
	"\xa1" => "\x7e",  # ~
	"\xa2" => "\x73",  # s
	"\xa3" => "\x74",  # t
	"\xa4" => "\x75",  # u
	"\xa5" => "\x76",  # v
	"\xa6" => "\x77",  # w
	"\xa7" => "\x78",  # x
	"\xa8" => "\x79",  # y
	"\xa9" => "\x7a",  # z
	"\xaa" => "",  # 
	"\xab" => "",  # 
	"\xac" => "",  # 
	"\xad" => "",  # 
	"\xae" => "",  # 
	"\xaf" => "",  # 
	"\xb0" => "",  # 
	"\xb1" => "",  # 
	"\xb2" => "",  # 
	"\xb3" => "",  # 
	"\xb4" => "",  # 
	"\xb5" => "",  # 
	"\xb6" => "",  # 
	"\xb7" => "",  # 
	"\xb8" => "",  # 
	"\xb9" => "\x60",  # `
	"\xba" => "",  # 
	"\xbb" => "",  # 
	"\xbc" => "",  # 
	"\xbd" => "",  # 
	"\xbe" => "",  # 
	"\xbf" => "",  # 
	"\xc0" => "\x7b",  # {
	"\xc1" => "\x41",  # A
	"\xc2" => "\x42",  # B
	"\xc3" => "\x43",  # C
	"\xc4" => "\x44",  # D
	"\xc5" => "\x45",  # E
	"\xc6" => "\x46",  # F
	"\xc7" => "\x47",  # G
	"\xc8" => "\x48",  # H
	"\xc9" => "\x49",  # I
	"\xca" => "",  # 
	"\xcb" => "",  # 
	"\xcc" => "",  # \int
	"\xcd" => "",  # 
	"\xce" => "",  # yen?
	"\xcf" => "",  # 
	"\xd0" => "\x7d",  # }
	"\xd1" => "\x4a",  # J
	"\xd2" => "\x4b",  # K
	"\xd3" => "\x4c",  # L
	"\xd4" => "\x4d",  # M
	"\xd5" => "\x4e",  # N
	"\xd6" => "\x4f",  # O
	"\xd7" => "\x50",  # P
	"\xd8" => "\x51",  # Q
	"\xd9" => "\x52",  # R
	"\xda" => "",  # 
	"\xdb" => "",  # 
	"\xdc" => "",  # 
	"\xdd" => "",  # 
	"\xde" => "",  # 
	"\xdf" => "",  # 
	"\xe0" => "\x5c",  # \
	"\xe1" => "",  # 
	"\xe2" => "\x53",  # S
	"\xe3" => "\x54",  # T
	"\xe4" => "\x55",  # U
	"\xe5" => "\x56",  # V
	"\xe6" => "\x57",  # W
	"\xe7" => "\x58",  # X
	"\xe8" => "\x59",  # Y
	"\xe9" => "\x5a",  # Z
	"\xea" => "",  # 
	"\xeb" => "",  # 
	"\xec" => "",  # h?
	"\xed" => "",  # 
	"\xee" => "",  # 
	"\xef" => "",  # 
	"\xf0" => "\x30",  # 0
	"\xf1" => "\x31",  # 1
	"\xf2" => "\x32",  # 2
	"\xf3" => "\x33",  # 3
	"\xf4" => "\x34",  # 4
	"\xf5" => "\x35",  # 5
	"\xf6" => "\x36",  # 6
	"\xf7" => "\x37",  # 7
	"\xf8" => "\x38",  # 8
	"\xf9" => "\x39",  # 9
	"\xfa" => "\x7c",  # | ou vide ?
	"\xfb" => "",  # 
	"\xfc" => "",  # 
	"\xfd" => "",  # 
	"\xfe" => "",  # 
	"\xff" => "",  # 
};

# ------------------------------------------------------------------------------------
# nom 		: cleanup_chars_as_table
# description 	: prend une chaine et nettoie ses caractres un par un en fonction 
# 		  de la table de traduction forunie
# parametres 	: - un mot 
# 		  - une table de correspondances en HREF de type 'char_ko' => 'char_ok'
# retours 	: le mot nettoy ou undef si erreur
# ------------------------------------------------------------------------------------
sub cleanup_chars_as_table($$)
{
	my ($word, $rh) = @_;
	return undef unless defined $word and defined $rh;

	# boucle sur les caractres du mot
	my @chars_ok;
	my @chars_ko = split //, $word;
	for (my $i=0; $i<scalar(@chars_ko); $i++) {
		my $c = $chars_ko[$i];
		if ( defined($rh->{$c}) ) {
			push @chars_ok, $rh->{$c};
		}
	}
	
	# le mot ne contient que des caractres autoriss selon la table fournie
	if ( $#chars_ok == -1 ) {
		return '';
	} else {
		return join '', @chars_ok;
	}
}

1;
