# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2006
# Authors and contributors:
# - Benjamin Bayart

package FDN::Common::Levenshtein;

use strict;
use Exporter;
use FDN::Common::Logger;

our @ISA = qw ( Exporter );
our @EXPORT = qw (
	levenshtein
	);

use warnings;

our $debug = 0;
sub __debug($) {
	my ( $msg ) = @_;
	debug $msg if ( $debug );
}

sub __debug_line($$) {
	return if ( ! $debug );
	my ( $prefix, $line ) = @_;
	my $dbg;
	$dbg = $prefix.' ';
	$dbg = join ' ',@$line;
	debug $dbg;
}

sub __debug_matrix($$$) {
	return if ( ! $debug );
	my ( $sa, $sb, $matrix ) = @_;
	my ( $tmpa, $tmpb );
	#$dbg = "  ";
	#$dbg .= join ' ',@$sb;
	#debug $dbg;
	__debug_line(' ',$sb);
	for $tmpa ( 0 .. $#$sa ) {
		__debug_line($sa->[$tmpa], $matrix->[$tmpa]);
#		$dbg = $sa->[$tmpa].' ';
#		for $tmpb ( 0 .. $lb-1 ) {
#			$dbg .= $matrix->[$tmpa]->[$tmpb].' ';
#		}
#		debug $dbg;
	}
}

sub levenshtein($$) {
	my ( $a, $b ) = @_;
	my $la = length($a);
	my $lb = length($b);
	if ( $la == 0 ) {
		return $lb;
	}
	if ( $lb == 0 ) {
		return $la;
	}
	my $cost = [];
	my $sa = [ split //, $a ];
	my $sb = [ split //, $b ];
	my ( $tmpa, $tmpb );
	for $tmpa ( 0 .. $la-1 ) {
		push @$cost, [];
		for $tmpb ( 0 .. $lb-1 ) {
			my $n = 0;
			$n = 1 if ( $sa->[$tmpa] ne $sb->[$tmpb] );
			push @{$cost->[$tmpa]}, $n;
		}
	}
	__debug "Matrice de cots (costs matrix):";
	__debug_matrix($sa, $sb, $cost);

	my $prev = [ $cost->[0]->[0] ];
	for $tmpb ( 1 .. $lb-1 ) {
		push @$prev, $prev->[$tmpb-1]+$cost->[0]->[$tmpb];
	}
	my $next  = [];
	__debug_line(' ',$sb);
	__debug_line($sa->[0], $prev);
	for $tmpa ( 1 .. $la-1 ) {
		push @$next, $prev->[0] + $cost->[$tmpa]->[0];
		for $tmpb ( 1 .. $lb-1 ) {
			my ( $left, $top, $diag, $best );
			$left = $next->[$tmpb-1] + 1;
			$best = $left;

			$top = $prev->[$tmpb] + 1;
			$best = $top if ( $top < $best );

			$diag = $prev->[$tmpb-1] + $cost->[$tmpa]->[$tmpb];
			$best = $diag if ( $diag < $best );

			__debug "Left: $left, top: $top, diag: $diag, best: $best";
			$next->[$tmpb] = $best;
		}
		$prev = $next;
		$next = [];
		__debug_line($sa->[$tmpa], $prev);
	}
	my $res = pop @$prev;
	__debug "Result: $res";
	return $res;
}

1;

