# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
# - Alexis Sukrieh
package FDN::Common::Logger;
##############################################################
# Logger.pm
# Gestion avance et paramtrable des logs des fdn-libs
##############################################################
##############################################################
# Initialisation du module
##############################################################
use strict;
use Exporter ;

our @ISA = ( 'Exporter' ) ;

our @EXPORT = qw (
	logger_prefix
	logger_init
	logger_close
	logger_config
	logger_purge
		
	debug
	info
	notice
	warning
	error
	critic
	alert

	start
	stop
		
	get_last_debug
	get_last_info
	get_last_notice
	get_last_warning
	get_last_error
	get_last_critic
	get_last_alert
	
	get_last_debug_msg
	get_last_info_msg
	get_last_notice_msg
	get_last_warning_msg
	get_last_error_msg
	get_last_critic_msg
	get_last_alert_msg
		
	%PILE_LOGS
);

use Sys::Syslog qw(:DEFAULT setlogsock); 
use File::Basename;
use FDN::Common::Constantes;

our %LAST_LOGS;				# une mmoire des derniers messages logus de chaque niveau
our %PILE_LOGS;				# la pile mmoire des messages logus
our $LOG_IS_ENABLED;			# le log est-il activ; initialisation dans le BEGIN
our $basename;				# par dfaut on utilise $0.
our $fullname;

my $config_default = {
	'use_memory'	=> 1,
	'use_syslog'	=> 1,
	'facility'	=> 'user'
};
my $config = $config_default;
sub USE_MEMORY { return $config->{'use_memory'}; }
sub USE_SYSLOG { return $config->{'use_syslog'}; }
sub DEFAULT_FACILITY { return $config->{'facillity'}; }


# on va maintenir un HRF pour les prefix selon les script appelant.
my $g_rh_prefix = {};


# renvoie le chemin complet vers le script appelant.
sub get_script_name() {
	# si on est en mod perl, il faut utiliser $ENV{'SCRIPT_FILENAME'}
	return $ENV{'SCRIPT_FILENAME'} if $ENV{'MOD_PERL'};
	return $0;
}

$fullname = get_script_name();
$basename = basename($fullname);


# ouverture de la socket unix vers syslog.
if (defined $ENV{'MOD_PERL'}) {
#	warn "Attention, en mod_perl, il faut utiliser logger_init() [".$ENV{'SCRIPT_FILENAME'}."]";
}
else {
	# On some versions of Sys::Syslog it slows down everything with no good reason...
	#setlogsock('unix');		
	openlog($basename, 'pid', DEFAULT_FACILITY);
}

# les flags particuliers du log
my $LOG_FLAGS	=	{
	debug	=>	TRUE,
	info	=>	TRUE,
	notice	=>	TRUE,
	warning =>	TRUE,
	error	=>	TRUE,
	critic 	=>	TRUE,
	alert	=>	TRUE
};

my %g_rh_label = (
	info    => 'info   ',
	notice  => 'notice ',
	error   => 'error  ',
	warning => 'warning',
	debug   => 'debug  ',
	critic  => 'critic ',
	alert   => 'alert  '
);





##############################################################
# mthodes publiques
##############################################################

sub logger_init() {
	return unless $ENV{'MOD_PERL'};

	# Same as above
	#setlogsock('unix');		
	$basename = basename($ENV{'SCRIPT_FILENAME'});
	$fullname = get_script_name();
	openlog($basename, 'pid', DEFAULT_FACILITY);
	logger_prefix("");
	delete $PILE_LOGS{$fullname} if defined $PILE_LOGS{$fullname};
	delete $LAST_LOGS{$fullname} if defined $LAST_LOGS{$fullname};
}

sub logger_close() {
	return unless $ENV{'MOD_PERL'};
	delete $PILE_LOGS{$fullname} if defined $PILE_LOGS{$fullname};
	delete $LAST_LOGS{$fullname} if defined $LAST_LOGS{$fullname};
	closelog();
}

sub logger_purge() {
	%PILE_LOGS = ();
	%LAST_LOGS = ();
}

# ------------------------------------------------------------------------------------
# nom : logger_prefix
#
# description : dfini un prefix qui sera utilis dans tous les messages envoy  
# 		syslog, trs pratique pour les site web.
#
# parametres : le message a utiliser comme prefixe.
# retours : void
# ------------------------------------------------------------------------------------
sub logger_prefix($) {
	my ($prefix) = @_;
	$prefix = "" unless defined $prefix;

	my $name = get_script_name();
	$g_rh_prefix->{$name} = $prefix;
	info("Prfixe de logs activ : '$prefix' pour le script $name");
}

# ------------------------------------------------------------------------------------
# nom : logger_config
#
# description : renseigne le nom de la configuration  utiliser
# 
# parametres : $nom
#
# retours :
# ------------------------------------------------------------------------------------
sub logger_config($) {
	my ( $name ) = @_;
	require FDN::Common::Config;
	local $FDN::Common::Config::debug = 0;
	my $common = FDN::Common::Config::Config_Section('logger.cfg', 'common');
	$common = {} if ! defined $common;
	my $local  = FDN::Common::Config::Config_Section('logger.cfg', $name);
	$local = {} if ! defined $local;
	$config = {
		%$config_default,
		%$common,
		%$local };
}

# ------------------------------------------------------------------------------------
# nom : start
#
# description : enregistre le debut d'une fonction
#
# parametres : @_ afin d'imprimer les variables forunies
#
# retours :
# ------------------------------------------------------------------------------------
sub start ($) {
	my $ra_args = shift();
	my @args = ();
	
	$ra_args = ["aucun arguments"] if (! ref($ra_args) eq 'ARRAY');
	
	my ($package, $filename, $line, $fonction) = caller (1);	

	for (my $i=0; $i<scalar(@{$ra_args}); $i++) {
		if (ref ($ra_args->[$i]) eq 'HASH') {
			
			my $h_tmp = $ra_args->[$i];
			my $string;
			
			$string = ("\t$i - {");
			while (my ($k, $v) = each (%{$h_tmp})) {
				$string .= "$k => $v, ";
			}
			chop($string);
			chop($string);
			$string .= "}\n";
			push (@args, $string);
		} else {
			push (@args, "\t$i - " . $ra_args->[$i] . "\n") if
			(defined ($ra_args->[$i]));
		}
	}
	
	&info ("$fonction (\n".join ("", @args)."\n)");
}



# ------------------------------------------------------------------------------------
# nom : stop
#
# description : enregistre la fin d'une fonction
#
# parametres :
#
# retours :
# ------------------------------------------------------------------------------------
sub stop () {
	my ($package, $filename, $line, $fonction) = caller (1);	
	&info ("\n[ STOP ]\n$fonction");
}


# ------------------------------------------------------------------------------------
# nom 		: debug
# description 	: log un message de debugguing
# parametres 	: message
# retours 	: TRUE ou FALSE selon le succs
# ------------------------------------------------------------------------------------
sub debug($)
{
	my ($message) = @_;
	return FALSE unless defined $message and length $message;
	return log_with_syslog('debug', $message);
}

sub info($) {
	my ($message) = @_;
	return FALSE unless defined $message and length $message;
	return log_with_syslog('info', $message);
}

sub notice($) {
	my ($message) = @_;
	return FALSE unless defined $message and length $message;
	return log_with_syslog('notice', $message);
}

sub warning($) {
	my ($message) = @_;
	return FALSE unless defined $message and length $message;
	return log_with_syslog('warning', $message);
}

sub error ($) {
	my ($message) = @_;
	return FALSE unless defined $message and length $message;
	return log_with_syslog('error', $message);
}

sub critic ($) {
	my ($message) = @_;
	return FALSE unless defined $message and length $message;
	return log_with_syslog('critic', $message);
}

sub alert ($) {
	my ($message) = @_;
	return FALSE unless defined $message and length $message;
	return log_with_syslog('alert', $message);
}


# ------------------------------------------------------------------------------------
# nom : get_last_info
# description : retourne un HREF dtaillant la dernire info loggue
# parametres : VOID
# retours :
# 	si une info au moins existe, retour du HREF de dtail de cette info :
#	{ 
# 	  MESSAGE => chaine du message
# 	  DATE 	  => date d'apparition du message
# 	  MODULE  => le module appelant
# 	  FONCTION=> la fonction appelant
# 	  FICHIER => le fichier Perl contenant l'appel
# 	  LIGNE   => la ligne o apparait l'appel
# 	}
# 	sinon renvoi -1
# ------------------------------------------------------------------------------------
sub get_last_debug () {
	return get_last_log('debug');
}

sub get_last_info () {
	return get_last_log ('info');
}

sub get_last_notice () {
	return get_last_log('notice');
}

sub get_last_warning () {
	return get_last_log('warning');
}

sub get_last_error () {
	return get_last_log('error');
}

sub get_last_critic () {
	return get_last_log('critic');
}

sub get_last_alert () {
	return get_last_log('alert');
}

# ------------------------------------------------------------------------------------
# nom : get_last_info_msg
#
# description : renvoie le dernier message d'info loggu
#
# parametres : 
#
# retours :
#	le message toruv en cas de succes
# 	en cas d'erreur on renvoie -1
# ------------------------------------------------------------------------------------
sub get_last_debug_msg () {
	return (get_last_log_msg ('debug'));
}

sub get_last_info_msg () {
	return (get_last_log_msg ('info'));
}

sub get_last_notice_msg () {
	return (get_last_log_msg ('notice'));
}

sub get_last_warning_msg () {
	return (get_last_log_msg ('warning'));
}

sub get_last_error_msg () {
	return (get_last_log_msg ('error'));
}

sub get_last_critic_msg () {
	return (get_last_log_msg ('critic'));
}

sub get_last_alert_msg () {
	return (get_last_log_msg ('alert'));
}


##############################################################
# methodes prives 
##############################################################


# ------------------------------------------------------------------------------------
# nom 		: log_with_syslog
# description 	: effectue le log en passant par syslog
# parametres 	: ($level, $message)
# 
# 			0 : le 'level' du message, parmi (par ordre de gravit selon syslog) 
# 				debug   : Message de debugging.
# 				info    : Message d'information simple.
# 				notice  : Condition normale, mais message significatif.
# 				warning : Des avertissement se prsentent.
# 				error   : Des erreurs se produisent.
# 				critic  : Les conditions sont critiques.
# 				alert   : Des actions doivent tre entreprises immdiatement.
#
#			2 : le message a logguer sans date 
# 				
# retours 	: TRUE si le log russi FALSE sinon
# ------------------------------------------------------------------------------------
sub log_with_syslog ($$) {
	my ($level, $message) = @_;
	return FALSE unless defined $level and defined $message;
	
	# dtermination du caller le plus pertinent 
	my $depth = 2;
	my ($package, $filename, $line, $fonction) = caller ($depth);
	while ( defined $fonction &&
		$fonction =~ /::(?:__)?(debug|info|warning|error|notice|alert|critic|return)/ ) {
		$depth++;
		($package, $filename, $line, $fonction) = caller ($depth);
	}

	##############################################################
	# vrification des arguments reus et initialisaiton
	############################################################## 
	$package  = "" unless defined $package;
	$filename = "" unless defined $filename;
	$line     = 0 unless defined $line;
	$fonction = $basename unless defined $fonction;
	$level = lc($level);
	$level = 'info' unless defined $level and length $level;
	return FALSE unless $LOG_FLAGS->{$level}; 
	
	# On va nettoyer le caller car modperl est trs imposant.
	if (defined $ENV{'MOD_PERL'}) {
		$fonction =~ s/_2e/./g;
		$fonction =~ s/_2d/-/g;
	}

	##############################################################
	# construction du message
	##############################################################
	unless (defined $message and length $message) { 
		$message = "[Aucun message]";
	}

	##############################################################
	# sauvegarde du message dans la pile mmoire des messages
	##############################################################
	if (USE_MEMORY) {
		my $log = {
			'msg'    => $message,
			'level'  => $level,
			'time'   => time(),
			'module' => $package,
			'line'   => $line,
			'file'   => $filename,
			'fct'    => $fonction
		};
		if ( ! exists $LAST_LOGS{$fullname} ) {
			$LAST_LOGS{$fullname} = {};
			$PILE_LOGS{$fullname} = [];
		}
		$LAST_LOGS{$fullname}->{$level} = $log;
		push @{$PILE_LOGS{$fullname}}, $log;
	}
	return TRUE if !USE_SYSLOG;

	my $level_str = $g_rh_label{$level};
	$message  = $level_str . " * $message";
	$message .= " - $fonction ($filename l. $line)" if $line;
	$message =~ s/%/%%/g;

	$message = $g_rh_prefix->{$fullname} . " > " . $message
		if  defined $g_rh_prefix->{$fullname}
		    and length  $g_rh_prefix->{$fullname};

	$level = 'err' if ( $level eq 'error' );
	$level = 'crit' if ( $level eq 'critic' );
	return syslog($level, $message);
}


# ------------------------------------------------------------------------------------
# nom : get_last_log
#
# description : renvoie un HREF contenant le dernier message loggu du type demand
# 		le module appelant, la date
# 		et la ligne a laquelle l'appel a t fait
#
# parametres : 
# 	- $_[0] : le type de log : info, warning ou error
#
# retours :
# 	en cas de succes : un HREF de la forme 
# 	{ 
# 	  MESSAGE => chaine du message
# 	  DATE 	  => date d'apparition du message
# 	  MODULE  => le module appelant
# 	  FONCTION=> la fonction appelant
# 	  FICHIER => le fichier Perl contenant l'appel
# 	  LIGNE   => la ligne o apparait l'appel
# 	}
#
# 	en cas d'erreur on renvoie 'undef'
#
# ------------------------------------------------------------------------------------
sub get_last_log (;$) {
	my ($type) = @_;
	
	my $log;
	if ( defined $type ) {
		$type = 'error' if $type eq 'err';
		$type = 'critic' if $type eq 'crit';
		$log = $LAST_LOGS{$fullname}->{$type};
	} elsif ( exists $PILE_LOGS{$fullname} && ref($PILE_LOGS{$fullname}) eq 'ARRAY' ) {
		my $id = $#{$PILE_LOGS{$fullname}};
		$log = $PILE_LOGS{$fullname}->[$id];
	}
	$log = { %$log } if defined $log;
	return $log;
}



# ------------------------------------------------------------------------------------
# nom : get_last_log_msg
#
# description : renvoie le dernier message loggu du type demand
#
# parametres : 
# 	- $_[0] : le type de log : info, warning ou error
#
# retours :
#	le message trouv en cas de succes
# 	en cas d'erreur on renvoie une chaine vide
# ------------------------------------------------------------------------------------
sub get_last_log_msg (;$) {
	my ($type) = @_;
	my $log = get_last_log($type);
	return '' if !defined $log;
	return $log->{'msg'};
}

END {
	closelog();
}


TRUE;


__END__

=head1 NOM

B<FDN::Common::Logger> 

=head1 SYNOPSYS

B<use FDN::Common::Logger;> (module Perl)

=head1 DESCRIPTION

B<Logger.pm> fournit une interface Perl pour syslog. Il est possible en utilisant 
ces fonctions de logguer simplement et efficacement des informations grce  syslog.

I<Note> : les termes B<facility> et B<level> employs dans cette doc sont ceux du vocabulaire syslog.
On reprendra ici par commodit les diffrentes valeurs possibles pour ces variables au sens syslog :

=head2 facility

I<Smantique> 

le type de programme appelant le loggueur, par dfaut : 'user'.

I<Valeurs> 

=over

=item cron    : message d'un dmon horaire (cron et at)

=item daemon  : dmon du systme sans classification particulire.

=item mail    : message du sous-systme de courier.

=item user    : message utilisateur gnrique.

=back

=head2 level

I<Smantique> 

dcrit le niveau de gravit du message, la liste est trie par ordre croissant selon syslog. 

I<Valeur>  

=over

=item debug   : Message de debugging.

=item info    : Message d'information simple.

=item notice  : Condition normale, mais message significatif.

=item warning : Des avertissements se prsentent.

=item error   : Des erreurs se produisent.

=item critic  : Les conditions sont critiques.

=item alert   : Des actions doivent tre entreprises immdiatement.

=back

=head2 Note

Par principe de commodit et de libert, nous avons implment une fonction pour chaque level existant. 
Cependant, nous recommandons l'usage des quatre fonctions documentes dans le prsent manuel : debug, info, warning et error pour 
viter une surcharge de varit de messages trop importante. Il en rsulterait une perte de pertinence et de lisibilit dans les 
fichiers de log.

On remarquera que les fonctions de log ont toutes la mme interface. Le seul argument est le message  logguer.

Enfin, on rappellera que l'on met en prfixe de chaque message loggu la date prcise  la seconde du message ainsi que le programme provoquant le message et son pid. Inutile donc de rajouter ces informations lors des appels  B<Logger.pm> !

=head1 FONCTIONS

=head2 logger_prefix ($prefixe);

Initialise un prefixe  utiliser pour chaque message loggu. Il sera ainsi possible de tracker dans le fichier de log
d'une manire simple un groupe d'appel au loggueur.
Cette fonction est surtout utile pour les sites web dcoups en plusieurs CGI distinct car dans ce cas,
le PID ne suffit plus  identifier le mme process.

Ex: 
    #!/usr/bin/perl
    use FDN::Common::Logger;
    logger_prefix("Contrat : ".$ARGV[0]);
    info "debut du traitement"

Ce qui provoquera l'output suivant dans le logger :

    Sep  7 17:39:38 artemis t.pl[11192]: info  * prefixe de logs activ : Contrat : 1234 - FDN::Common::Logger::logger_prefix (/tmp/test.pl l. 6)
    Sep  7 17:39:38 artemis t.pl[11192]: Contrat : 1234 > info  * debut du traitement


=head2 debug ($message);

Gnre un message de level I<debug>.

A utiliser pour logguer des informations de dbuggage. 
Typiquement, la valeur d'un compteur  un moment donn, 
des messages de traage, ou toute autre information peu pertinente en dehors du moment ou l'on code (en cas de doute
se demander si l'info sera comprhensible sortie du contexte).

Les messages de debug sont prvus pour tre nombreux, ne pas hsiter  en abuser !

=head2 info ($message);

Gnre un message de level I<info>.

A utiliser pour stocker des informations sur le droulement normal d'un processus. 
Typiquement, informer que le contrat numro 1234 s'est authentifi  14:23 sur le tableau de bord. 
Prendre garde  utiliser avec parcimonie les messages de level info. Il faut que la lecture des logs soit 
possible pour une personne tierce sans connatre le code.

=head2 warning ($message);

Gnre un message de level I<warning>.

A utiliser dans le cas ou un comportement anormal mais I<non bloquant> se prsente.
Typiquement, un argument optionnel est fourni mais son format est incorrect, l'envoi d'un mail a chou.

=head2 error ($message);

Gnre un message de level I<error>.

A utiliser en cas d'erreur rellement bloquante.
Typiquement les arguments obligatoires sont manquant, la base de donnes est inaccessible, 
le fichier de template du site web n'est pas lisible, etc.



=head1 COMPORTEMENT

Tous les messages gnrs par B<Logger.pm> seront passs  syslog via une socket de type B<unix>. Les ouvertures de fichiers ne sont donc pas fates directement par le processus utilisant B<Logger.pm> mais bien par le dmon B<syslogd>.

Le comportement des fichiers de logs est donc directement dpendant de la configuration de syslog. Nous rsumerons ici les principales particularits de syslog dans un cas de configuration par dfaut sous un systme B<debian>.

Par dfaut, tous les messages de B<facility> I<user> seront loggus dans /var/log/syslog et dans /var/log/user.log.
Il serait pertinent de ne pas logguer sur les machines de production les messages de B<level> I<debug> mais uniquement sur les machines de dveloppement.

Voici le type de messages que l'on peux rencontrer dans user.log avec une configuration conventionnelle :

    Mar 23 16:46:23 artemis logger.t[26357]: debug * voici un premier message 
    Mar 23 16:46:23 artemis logger.t[26357]: info  * je vais vous dire quelque chose ici qui me passae par la tete 
    Mar 23 16:46:23 artemis logger.t[26357]: note  * attention, il faut penser  utiliser cela 
    Mar 23 16:46:23 artemis logger.t[26357]: warn  * bizare, on a pas trouver de valeur dans le champ de mas 
    Mar 23 16:46:23 artemis logger.t[26357]: error * comment faire sans variable de type couscous ? 
    Mar 23 16:46:23 artemis logger.t[26357]: crit  * il n'y a plus de harissa ! 
    Mar 23 16:46:23 artemis logger.t[26357]: alert * je ne peux plus continuer de la sorte, c'en est trop!  
    Mar 23 16:46:23 artemis logger.t[26357]: info  * la fonction fait une info - main::fonction (t/logger.t l. 21) 
    Mar 23 16:46:23 artemis logger.t[26357]: warn  * la fonction fait un warning - main::fonction (t/logger.t l. 21) 

=head1 EXAMPLES

    #!/usr/bin/perl
    use strict;
    use warnings;

    use FDN::Common::Logger;
    info "dmarage du programme a";
    error "on a provoqu une erreur" if ($ARGV[0] eq 'error');

=head1 SEE ALSO

Sys::Syslog, syslog(3), syslogd


=head1 AUTEURS

Benjamin Bayart <bayartb@edgard.fdn.fr>

Alexis Sukrieh <alexis@sitadelle.com>

Thomas Parmelan <tom@sitadelle.com>


=cut

