package FDN::Common::Sign;

use Encode;
use MIME::Base64;
use Digest::SHA qw(sha1 sha1_hex);
use Crypt::DH 0.05;
use base 'Exporter';

our @EXPORT = qw (
	sign_b64
	sign_hex
);


sub sign_b64($$) {
	my ( $info, $secret ) = @_;
	if ( Encode::is_utf8($info) ) {
		Encode::_utf8_off($info);
	}
	return _b64(hmac_sha1($info, $secret));
}

sub sign_hex($$) {
	my ( $info, $secret ) = @_;
	if ( Encode::is_utf8($info) ) {
		Encode::_utf8_off($info);
	}
	return hmac_sha1_hex($info, $secret);
}


sub _b64 {
    my $val = MIME::Base64::encode_base64($_[0]);
    $val =~ s/\s+//g;
    return $val;
}


sub hmac_sha1_hex {
    unpack("H*", &hmac_sha1);
}
sub hmac_sha1 {
    hmac($_[0], $_[1], \&sha1, 64);
}
sub hmac {
    my($data, $key, $hash_func, $block_size) = @_;
    $block_size ||= 64;
    $key = &$hash_func($key) if length($key) > $block_size;

    my $k_ipad = $key ^ (chr(0x36) x $block_size);
    my $k_opad = $key ^ (chr(0x5c) x $block_size);

    &$hash_func($k_opad, &$hash_func($k_ipad, $data));
}

1;

