# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
# - Alexis Sukrieh
# - Christophe Caillet
package	FDN::Common::Validate ;

use strict;
use Exporter ;

our @ISA = ( 'Exporter' ) ;

our @EXPORT	= qw (
	Is_Domain
	Is_RFC822Domain
	Is_Email
	Is_RFC822Email
	Is_IPv4
	Is_Public_IPv4
	Is_RR_Name
	Is_RR_Ttl
	Is_RR_Type
	Is_RR_Data
	Is_NAF_Code
	Is_SIREN_Code
	Is_SIRET_Code
	Is_INSEE_Code
	Is_Unix_User
	Is_YYYYMMDD_Date
	Is_HHMMSS_Time
	Is_TimestampDB
	Is_Timestamp14
	Is_Timestamp8
	Is_Netmask
	Is_Boolean
	Is_Number
	Is_Integer
	Is_Vlan
	Is_Vrid
	Is_Ipv4
	Is_Port
	Is_Port_Range
	CIDR_SHORT
	CIDR_LONG
	Is_Cidr
	Is_Host
	Is_Mib_Oid
	Is_Member
);

use warnings;

use Date::Format;
use Net::CIDR;
use FDN::Common::Constantes;

# Regexp concernant les domaines et les adresses mails
# La definition des regexp de domain et de mail est conforme au RFC 1035 pour le nom de domaine
# et RFC 822 pour le mail. Pour plus de details se reporter a ces RFC.
my $RFC_DOMAIN_LENGTH	= 255 ;
# liste de TLD attribué : http://data.iana.org/TLD/tlds-alpha-by-domain.txt
# Version 2015083101, Last Updated Tue Sep  1 07:07:01 2015 UTC
my $OLD_TLD = 'xn--deba0ad|eh|xn--9t4b11yi5a|xn--0zwm56d|tp|um|xn--11b5bs3a9aj6g|xn--hgbk6aj7f53bba|xn--kgbechtv|xn--zckzah|an|xn--hlcj6aya9esc7a|yu|xn--jxalpdlp|xn--80akhbyknj4f|xn--g6w251d';
my $CUR_TLD = 'nu|band|lv|rs|ceo|nico|au|audio|lancaster|fly|am|mr|everbank|kh|holdings|forex|xn--3ds443g|gh|mg|school|lupin|oracle|mz|lexus|attorney|montblanc|reisen|realtor|coach|mma|as|zuerich|et|xn--cg4bki|republican|durban|cards|mobi|ubs|coop|markets|show|yandex|tattoo|training|xn--wgbl6a|tn|immobilien|xn--fjq720a|xn--czr694b|va|sa|ao|soccer|bible|gent|rio|thd|java|arpa|xn--nqv7fs00ema|sz|computer|mq|cbn|sandvik|lt|center|wales|cisco|immo|ice|study|parts|bj|farm|xperia|golf|wtf|xn--3e0b707e|xn--ngbc5azd|online|rocks|google|xn--clchc0ea0b2g2a9gcd|bzh|gn|fage|claims|swatch|gifts|scor|trading|fish|sakura|ky|datsun|id|soy|world|xn--efvy88h|nf|nissan|paris|cab|gd|social|physio|chloe|cfa|axa|ryukyu|xn--nqv7f|engineer|vn|ng|events|tt|hitachi|barclays|homedepot|solar|supplies|auction|bradesco|infiniti|consulting|top|prod|market|xn--3pxu8k|citic|mtn|gmx|town|ro|bbva|me|play|doosan|gallery|vodka|dj|mini|dnp|archi|international|xn--czrs0t|budapest|xn--mk1bu44c|jm|xn--9dbq2a|xn--90a3ac|dance|boutique|il|homes|airforce|sky|mo|vacations|jcb|legal|ae|fitness|money|cymru|skype|starhub|bank|careers|ren|forsale|okinawa|abbott|dclk|care|sy|xn--l1acc|genting|cx|xn--45q11c|day|degree|tennis|xn--fpcrj9c3d|er|company|glass|pub|hotmail|lr|xin|om|xn--d1alf|ck|diet|hangout|futbol|latrobe|at|xn--wgbh1c|life|hsbc|toray|ch|np|coffee|vg|guitars|ws|nyc|bh|md|house|ve|xn--hxt814e|cy|bt|aco|mx|co|cal|ge|joburg|expert|bloomberg|mango|lixil|college|sony|lu|gy|xn--j6w193g|gratis|corsica|live|army|aw|tv|xn--9et52u|gift|kim|motorcycles|photography|lgbt|xn--qcka1pmc|mc|lasalle|nec|gov|xn--55qw42g|fishing|docs|berlin|recipes|yodobashi|luxury|democrat|bz|erni|xn--kpry57d|temasek|forum|ninja|ventures|gold|xn--c1avg|dz|stcgroup|cricket|exchange|kyoto|pt|construction|bridgestone|dk|cooking|fit|yt|memorial|business|mu|xn--j1amh|tm|dabur|tax|sca|juegos|xn--vuq861b|piaget|bargains|bayern|pharmacy|ruhr|de|tk|honda|gal|law|directory|jewelry|xn--ygbi2ammx|software|group|amica|madrid|saarland|meme|xn--io0a7i|cr|email|aquarelle|deals|ee|trade|istanbul|brother|philips|az|goog|ir|mn|navy|ne|vista|versicherung|sohu|bo|ing|plus|ls|tl|tirol|gf|za|delivery|rip|xn--i1b6b1a6a2e|tickets|fj|xn--90ais|foundation|mtpc|sydney|xn--vermgensberatung-pwb|reviews|xn--t60b56a|viva|iwc|repair|best|institute|th|xn--xhq521b|direct|singles|financial|xn--kput3i|xn--flw351e|movistar|foo|jll|here|ps|sj|site|rest|credit|fyi|photo|travel|realty|cancerresearch|no|ba|today|vision|name|cv|pg|es|xn--mgbaam7a8h|fo|vistaprint|desi|football|technology|do|sarl|qpon|tires|biz|williamhill|sucks|engineering|xn--xkc2al3hye2a|cl|services|management|rent|ug|abogado|camp|solutions|lk|yachts|tips|stc|xn--fiq228c5hs|sk|sandvikcoromant|clothing|xn--mgbc0a9azcg|xn--80ao21a|nexus|eat|microsoft|place|flights|cf|net|creditcard|cn|moscow|luxe|schule|cern|lighting|investments|tui|vegas|goldpoint|xn--c2br7g|xn--rhqv96g|la|commbank|xn--fzc2c9e2c|dad|ovh|country|quebec|feedback|dentist|pk|men|hiv|vc|xn--h2brj9c|asia|bs|frl|xxx|pf|lat|dm|bm|builders|amsterdam|fm|ye|hosting|sex|irish|lacaixa|broker|fan|ke|voto|af|lotte|family|my|cw|apartments|delta|lc|drive|academy|jprs|pa|buzz|tatar|youtube|ga|accenture|sx|hermes|xn--mgbpl2fh|works|xn--45brj9c|panerai|crown|xn--p1acf|schmidt|menu|aq|netbank|redstone|liaison|xn--yfro4i67o|finance|pictet|xn--mgbayh7gpa|wf|xn--o3cw4h|clinic|orange|xn--nyqy26a|weir|network|kz|contractors|xn--fiqz9s|guge|xn--p1ai|vet|xn--mgb9awbf|tel|wtc|bauhaus|si|xn--czru2d|courses|suzuki|eu|ar|sg|xn--42c2d9a|wine|ntt|bing|schwarz|koeln|uno|bike|vin|ie|career|casa|bd|ms|uz|alsace|ibm|equipment|xn--unup4y|dating|enterprises|pink|goo|pet|kr|cartier|furniture|bharti|club|actor|barclaycard|bb|monash|jp|movie|bar|film|page|condos|reit|click|one|fans|flowers|iinet|capetown|bingo|rw|church|xn--lgbbat1ad8j|hoteles|cash|mormon|ad|limited|xn--tckwe|ooo|domains|kred|ngo|express|pro|man|miami|mh|productions|wien|firmdale|xn--fiq64b|gg|itau|tw|sr|boots|crs|cu|ci|xn--30rr7y|ai|android|garden|rentals|lotto|rsvp|catering|xn--kprw13d|bg|chat|lol|horse|ca|gb|statoil|srl|mm|frogans|channel|zw|edu|moe|pn|style|adult|download|dvag|uy|meet|digital|christmas|office|nz|giving|fund|pohl|support|xn--mgbbh1a71e|exposed|beer|rehab|build|to|sv|kn|partners|agency|xbox|date|caravan|tf|capital|gq|gl|host|accountant|ph|tokyo|pl|tours|io|review|supply|canon|racing|villas|university|otsuka|so|telefonica|uol|org|fk|haus|lb|autos|cba|surf|bond|gw|xn--80asehdb|yokohama|jetzt|restaurant|hn|samsung|xn--gecrj9c|party|systems|cyou|trust|cc|na|br|guru|ricoh|mt|krd|press|ifm|wiki|mil|xn--6frz82g|pr|xn--mgberp4a5d4ar|re|game|xn--j1aef|xn--3bst00m|ru|xn--d1acj3b|kw|xn--vermgensberater-ctb|ly|cuisinella|pics|cfd|xn--mgbx4cd0ab|sd|mv|coupons|link|ong|jo|xn--ses554g|discount|sap|blackfriday|whoswho|marriott|sm|bentley|tatamotors|shiksha|vote|yoga|globo|watch|cz|lease|dog|ipiranga|ni|organic|zm|airtel|nrw|london|int|sener|ist|it|loan|property|ski|nr|xn--fhbei|sn|cool|cars|allfinanz|city|xyz|rodeo|cm|education|icbc|studio|aaa|ec|wme|wang|bcn|post|be|help|tr|graphics|cafe|xn--kcrx77d1x4a|bet|properties|al|spiegel|ac|cheap|abb|onl|energy|hu|gdn|loans|uk|xn--80adxhks|sncf|gripe|tc|dental|science|cd|marketing|xn--55qx5d|xn--pssy2u|ua|tz|jlc|nl|melbourne|gmail|tienda|hockey|vi|cg|li|ma|pe|photos|team|codes|gr|xn--vhquv|hk|holiday|insure|xn--node|video|jobs|nra|fi|toys|xn--s9brj9c|td|black|prof|accountants|florist|pm|nagoya|spreadbetting|pictures|mortgage|gs|zip|leclerc|xn--zfr164b|green|guide|webcam|barcelona|kitchen|tg|voyage|run|hm|kaufen|surgery|gt|xn--mgba3a4f16a|dev|sh|bnpparibas|diamonds|bw|reise|bbc|porn|auto|su|moda|mba|xn--b4w605ferd|gp|mw|gi|cat|wedding|us|windows|xn--mxtq1m|aig|fail|museum|gu|scot|vlaanderen|xn--xkc2dl3a5ee0h|brussels|gm|hiphop|flsmidth|maison|win|swiss|mk|qa|tech|osaka|praxi|land|xn--estv75g|sale|shoes|industries|gle|bv|gmo|faith|im|pw|media|earth|ceb|bi|sb|kiwi|toyota|lds|sc|report|sl|kp|gop|associates|theater|work|pizza|aero|how|tj|estate|bio|gives|seek|healthcare|azure|vu|fr|aeg|space|is|km|design|community|limo|boo|seat|icu|emerck|shriram|taipei|afl|toshiba|ggee|boats|cologne|app|eg|st|red|lidl|xn--6qq986b3xl|xerox|ht|mp|zone|ax|walter|sexy|bid|sew|news|komatsu|gbiz|casino|se|eus|xn--11b4c3d|cruises|py|xn--4gbrim|mov|saxo|love|scb|ki|cloud|xn--q9jyb4c|xn--imr513n|ml|blue|com|gea|bn|hamburg|scholarships|viajes|nc|epson|active|rich|nhk|wed|hr|bnl|maif|xn--80aswg|kddi|doha|plumbing|ads|ink|xn--y9a3aq|website|xn--ogbpf8fl|iq|voting|camera|xn--mgbab2bd|nadex|xn--fiqs8s|chrome|kg|fashion|cleaning|by|je|neustar|xn--1qqw23a|tools|taxi|new|eurovision|ltda|bf|sanofi|esq|ag|global|mom|omega|chanel|nokia|bmw|lawyer|poker|in|xn--pgbs0dh|info';
our $TLD = "($CUR_TLD)";
our $ALL_TLD = "($CUR_TLD|$OLD_TLD)";
our $LABEL_DOMAIN		= '[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?' ;
our $PATTERN_DOMAIN		= '('.$LABEL_DOMAIN.'\.)+'.$TLD ;
our $PATTERN_RFC822DOMAIN	= '('.$LABEL_DOMAIN.'\.)*'.$LABEL_DOMAIN ;
our $FQDN			= $PATTERN_DOMAIN.'\.' ;
my $DIG_CMD = '/usr/bin/dig';

our $LOCAL_WORD_MAIL		= '[a-zA-Z0-9](?:[a-zA-Z0-9_-]*[a-zA-Z0-9])?' ;
our $LOCAL_PART_MAIL		= $LOCAL_WORD_MAIL.'(\.'.$LOCAL_WORD_MAIL.')*(\+'.$LOCAL_WORD_MAIL.'(\.'.$LOCAL_WORD_MAIL.')*)?' ;
our $PATTERN_MAIL		= $LOCAL_PART_MAIL.'\@'.$PATTERN_DOMAIN ;
our $PATTERN_RFC822MAIL		= $LOCAL_PART_MAIL.'\@'.$PATTERN_RFC822DOMAIN ;

# Regexp adressage reseau
our $PATTERN_IPV4		= '([\d]{1,3}\.){3}[\d]{1,3}' ;
our $PATTERN_CIDRV4		= $PATTERN_IPV4.'\/[\d]{1,2}' ;
my $LABEL_IPV6			= '[\dA-F]{1,4}' ;
our $PATTERN_IPV6_FULL		= '('.$LABEL_IPV6.')(:'.$LABEL_IPV6.'){7}' ;

# Regexp relative au DNS
our $PATTERN_RR_NAME		= '(([a-zA-Z0-9\*](([a-zA-Z0-9\-\*]+)?[a-z0-9\*])?(\.)?)+|\@)' ;
our $PATTERN_RR_TYPE		= '(A|CNAME|NS|MX( [\d]+)?|PTR)' ;
our $PATTERN_RR_TTL		= '([\d]+[dhwsDHWS]?)?' ;
our $PATTERN_RR_DATA		= {} ;
$PATTERN_RR_DATA->{'A'}		= $PATTERN_IPV4 ;
$PATTERN_RR_DATA->{'CNAME'}	= '('.$LABEL_DOMAIN.'(\.'.$LABEL_DOMAIN.')*|'.$FQDN.')' ;
$PATTERN_RR_DATA->{'MX'}	= '('.$LABEL_DOMAIN.'(\.'.$LABEL_DOMAIN.')*|'.$FQDN.')' ;
$PATTERN_RR_DATA->{'NS'}	= '('.$LABEL_DOMAIN.'(\.'.$LABEL_DOMAIN.')*|'.$FQDN.')' ;
$PATTERN_RR_DATA->{'PTR'}	= $FQDN ;

# Regexp relative aux codes administratifs divers
our $PATTERN_NAF_CODE	= '[\d]{3}[A-Z]' ;
our $PATTERN_SIREN_CODE	= '[\d]{9}' ;
our $PATTERN_SIRET_CODE	= '[\d]{14}' ;
our $PATTERN_INSEE_CODE	= '[\d]{5}' ;

# Regexp relative au timestamp
our $TIMESTAMP14		= '[\d]{14}' ;
our $TIMESTAMP8			= '[\d]{8}' ;
my $BEGIN_YEAR			= 1900 ;
my $CURRENT_YEAR		= time2str ( "%Y", time ) ;
my $BEGIN_MONTH			= 1 ;
my $END_MONTH			= 12 ;
my $BEGIN_DAY			= 1 ;
my $END_DAY			= 31 ;
my $BEGIN_HOUR			= 0 ;
my $END_HOUR			= 23 ;
my $BEGIN_MIN			= 0 ;
my $END_MIN			= 59 ;


# Desc	: Verifie que le domaine $domain est conforme a la RFC1035, renvoie 1 si oui et 0 sinon
# Proto	: BOOL Is_Domain ( STR $domain ) ;
# Parametres	: $domain est le nom de domaine a controler
sub Is_Domain($) {
	my ( $domain ) = @_ ;
	return FALSE if ( length ( $domain ) > $RFC_DOMAIN_LENGTH );
	return FALSE if ( $domain !~ /^$PATTERN_DOMAIN$/i );
	return TRUE;
}

sub Is_RFC822Domain ($) {
	my ( $domain ) = @_;
	return FALSE if ( length ( $domain ) > $RFC_DOMAIN_LENGTH );
	return FALSE if ( $domain !~ /^$PATTERN_RFC822DOMAIN$/i );
	return TRUE;
}

# Desc	: Verifie que l'adresse $mail est RFC822 compatible et si $real = 1
#	  verifie qu'au moins un MX est declare pour le domaine, renvoie 1 si
#	  tout est ok et 0 sinon.
#	  N'accepte que les domaines RFC1035 et pas tous les domaines RFC822.
# Proto	: BOOL Is_Email ( STR $mail, BOOL $real ) ;
# Parametres	: $mail est l'adresse mail a verifier
#		: $real specifie si un controle d'existence de MX doit etre fait ou non
sub Is_Email($;$) {
	my ( $mail, $real ) = @_;
	return __is_email($mail, $PATTERN_MAIL, $real);
}

# Idem, mais accepte les domaines mails qui ne sont pas des domaines DNS (sauf
# dans le cas ou $real est vrai, dans ce cas les les deux fonctions sont
# identiques).
sub Is_RFC822Email($;$) {
	my ( $mail, $real ) = @_;
	return __is_email($mail, $PATTERN_RFC822MAIL, $real);
}

sub __is_email($$;$) {
	# Parametres, variables et constantes locales
	my ( $mail, $pattern, $real ) = @_ ;

	return FALSE if (! defined $mail);
	return FALSE if ($mail eq '');

	$real = 0 if (! defined $real);

	my ( $command, $return_code, @output ) ;

	my ($prefix, $domain) = split ( /\@/, $mail ) ;
	if ( length ( $prefix ) > 100 ) {
		return FALSE;
	} elsif ( $mail !~ /^$pattern$/ ) {
		return FALSE;
	} elsif ( $real ) {
		$command	= "$DIG_CMD $domain +noauthority +noadditional" ;
		@output		= `$command` ;
		$return_code	= $? >> 8 ;
		if ( $return_code ) {
			return FALSE;
		} else {
			foreach ( @output ) {
				chomp ;
				if ( /HEADER[^,]+, status:\s*([^,]+),.*$/ ) {
					# Le domain est inexistant
					if ( $1 eq 'NXDOMAIN' ) {
						return FALSE;
					}
				} elsif ( /ANSWER: ([\d]+),/ ) {
					# Nombre de reponse pour le domaine i.e. champs mx
					if ( ! $1 ) {
						return FALSE;
					} else {
						return TRUE;
					}
				}
			}
		}
	}
	return TRUE;
}

# Gestion des validations DNS
# Les protocoles sont case-insensitive donc tous controle de regexp seront avec le flag i

sub Is_IPv4($);
sub Is_IPv4($) {
	my ( $ip ) = @_ ;

	defined $ip or return 0;

	if ( $ip =~ /\// ) {
		return FALSE if ( $ip !~ /^$PATTERN_CIDRV4$/ );
		my ( $prefix, $cidr ) = split ( /\//, $ip ) ;
		return FALSE if ( ! Is_IPv4 ( $prefix ) );
		return FALSE if ( $cidr > 32 );
	} else {
		return FALSE if ( $ip !~ /^$PATTERN_IPV4$/ );
		my @mask = split ( /\./, $ip ) ;
		foreach ( @mask ) {
			return FALSE if ( $_ > 255 );
		}
	}
	return TRUE;
}

sub Is_Public_IPv4($) {
	my ($ip) = @_ ;

	return FALSE if ( !Is_IPv4 ($ip) );

	if (Net::CIDR::cidrlookup ($ip, (
		'0.0.0.0/8',      # broadcast network
		'127.0.0.0/8',    # loopback network
		'224.0.0.0/4',    # class D multicast network
		'240.0.0.0/5',    # class E reserved network
		'248.0.0.0/5',    # reserved network

		'10.0.0.0/8',     # rfc1918 private network
		'172.16.0.0/12',  # rfc1918 private network
		'192.168.0.0/16', # rfc1918 private network

		'169.254.0.0/16', # LINK-LOCAL network
		'192.0.2.0/24',   # TEST-NET network

		'1.0.0.0/8',      # US army
		))) {
			return FALSE;
	}

	return TRUE;
}

# arguments :
#	- $mask : un masque de type CIDR (0..32) ou un netmask de la
#	  forme 255.255.0.0
# valeur de retour :
#	- 1 si c'est un masque valide
#	- 0 sinon
sub Is_Netmask($) {
	my ( $mask ) = @_;
	my $prev = 255;
	defined $mask or return FALSE;
	if ( $mask =~ /^\d{1,2}$/ and
	     $mask <= 32 ) {
		return TRUE;
	}
	return FALSE if ( $mask !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/ );
	foreach my $octet ($1, $2, $3, $4) {
		if ( $prev != 255 && $octet != 0 ) {
			return FALSE;
		}
		if ( !grep ($_ == $octet, (255,254,252,248,240,224,192,128,0)) ) {
			return FALSE;
		}
		$prev = $octet;
	}
	return TRUE;
}

sub Is_RR_Ttl($) {
	my ( $rr_ttl ) = @_ ;

	return FALSE if ( $rr_ttl !~ /^$PATTERN_RR_TTL$/ );
	return TRUE;
}

# Desc	: Verifie que le nom de ressource $rr_name est RFC1035 compatible,
#	  renvoie 1 si oui et 0 sinon
# Proto	: BOOL Is_RR_Name ( STR $rr_name, STR $zone ) ;
# Parametres	: $rr_name est le nom de la ressource a controler
#		: $zone est le nom de la zone a laquelle la ressource appartient
#		  si celui-ci est connu
sub Is_RR_Name($;$) {
	my ( $rr_name, $zone ) = @_ ;

	if ( defined $zone && Is_Domain ( $zone ) ) {
		# Le nom est un FQDN mais ne concorde pas avec le nom de la zone
		return FALSE if ( $rr_name =~ /\.$/ && $rr_name !~ /$zone/ );
		if ( $rr_name =~ /$zone/ ) {
			# Le nom la zone apparait dans $rr_name on la supprime pour
			# tester uniquement le nom de la resource
			$rr_name =~ s/$zone.*$// ;
		}
	}
	return FALSE if ( $rr_name !~ /^$PATTERN_RR_NAME$/i );
	return TRUE;
}

# Desc	: Verifie que le type de ressource est RFC1035 compatible, renvoie 1 si oui et 0 sinon
# Proto	: BOOL Is_RR_Type ( STR $rr_type ) ;
# Parametres	: $rr_type est le type de la ressource a controler
sub Is_RR_Type($) {
	my ( $rr_type ) = @_ ;
	return FALSE if ( $rr_type !~ /^$PATTERN_RR_TYPE$/ );
	return TRUE;
}

# Desc	: Verifie que la data $rr_data d'une ressource de type $rr_type est RFC1035
# 	  compatible, renvoie 1 si oui et 0 sinon
# Proto	: BOOL Is_RR_Data ( STR $rr_data, STR $rr_type ) ;
# Parametres	: $rr_data est la data de ressource a controler
#		: $rr_type est le type de la ressource
sub Is_RR_Data($$) {
	my ( $rr_data, $rr_type ) = @_ ;

	return FALSE if ( ! Is_RR_Type ( $rr_type ) );
	$rr_type =~ s/^MX.*$/MX/;
	$rr_type = uc $rr_type;
	my $pattern = $PATTERN_RR_DATA->{$rr_type};
	return FALSE if ( $rr_data !~ /^$pattern$/i );
	return FALSE if ( $rr_data =~ /^$PATTERN_IPV4$/ && ! Is_IPv4 ( $rr_data ) );
	return TRUE;
}

sub Is_NAF_Code($) {
	my ( $code ) = @_ ;
	return FALSE if ( $code !~ /^$PATTERN_NAF_CODE$/ );
	return TRUE;
}

sub Is_SIREN_Code($) {
	my ( $code ) = @_ ;
	return FALSE if ( $code !~ /^$PATTERN_SIREN_CODE$/ );
	return TRUE;
}

sub Is_SIRET_Code($) {
	my ( $code ) = @_ ;
	return FALSE if ( $code !~ /^$PATTERN_SIRET_CODE$/ );
	return TRUE;
}

sub Is_INSEE_Code($) {
	my ( $code ) = @_ ;
	return FALSE if ( $code !~ /^$PATTERN_INSEE_CODE$/ );
	return TRUE;
}

sub Is_Unix_User($) {
	my ( $user ) = @_ ;
	return FALSE if ( ! defined ( getpwnam ( $user ) ) );
	return TRUE;
}

sub Is_YYYYMMDD_Date($) {
	my ( $date ) = @_ ;

	my ( $year, $month, $day );
	if ( $date =~ /^(\d{4})(\d\d)(\d\d)$/ ) {
		( $year, $month, $day ) = ( $1, $2, $3 );
	} else {
		return FALSE;
	}

	return FALSE if ( $year < $BEGIN_YEAR || $year > $CURRENT_YEAR );
	return FALSE if ( $month < $BEGIN_MONTH || $month > $END_MONTH );
	return FALSE if ( $day < $BEGIN_DAY || $day > $END_DAY );
	return TRUE;
}

sub Is_HHMMSS_Time($) {
	my ( $time ) = @_ ;
	my ( $hour, $min, $sec ) ;
	
	if ( $time =~ /^(\d\d)(\d\d)(\d\d)$/ ) {
		( $hour, $min, $sec ) = ( $1, $2, $3 );
	} else {
		return FALSE;
	}

	return FALSE if ( $hour < $BEGIN_HOUR || $hour > $END_HOUR );
	return FALSE if ( $min  < $BEGIN_MIN  || $min  > $END_MIN  );
	return FALSE if ( $sec  < $BEGIN_MIN  || $sec  > $END_MIN  );
	return TRUE;
}

sub Is_TimestampDB($) {
	my ( $ts ) = @_;
	my ( $date, $time );
	if ( $ts =~ /^(\d{4})-(\d\d)-(\d\d)\s+(\d\d):(\d\d):(\d\d)$/ ) {
		$date = "$1$2$3";
		$time = "$4$5$6";
	} else {
		return FALSE;
	}
	return FALSE if ( ! Is_YYYYMMDD_Date($date) );
	return FALSE if ( ! Is_HHMMSS_Time($time) );
	return TRUE;
}

sub Is_Timestamp14($) {
	my ( $timestamp ) = @_ ;
	my ( $date, $time ) ;

	if ( $timestamp =~ /^(\d{8})(\d{6})$/ ) {
		( $date, $time ) = ($1, $2);
	} else {
		return FALSE;
	}

	return FALSE if ( ! Is_YYYYMMDD_Date($date) );
	return FALSE if ( ! Is_HHMMSS_Time($time) );
	return TRUE;
}

sub Is_Timestamp8($) {
	return Is_YYYYMMDD_Date($_[0]);
}

# Cette fonction prend un booleen sous diverses formes en parametre.  Si
# ce n'est pas un booleen, la fonction retourne `undef' sinon elle retourne
# la valeur numerique du booleen (0 ou 1).
# Note: La fonction ne tient pas compte de la casse.
# ex:	Entree		Sortie
#	"yes"		1
#	"true"		1
#	"y"		1
#	"on"		1
#	1		1
#	"no"		0
#	"false"		0
#	"n"		0
#	"off"		0
#	0		0
#	2		undef
#	"peut-etre"	undef
sub Is_Boolean($) {
    my ($boolean) = @_;
    my %map = ( 'yes'   => TRUE,  'true'  => TRUE,  'y'     => TRUE,
		'on'    => TRUE,  '1'     => TRUE,
		'no'    => FALSE, 'false' => FALSE, 'n'     => FALSE,
		'off'   => FALSE, '0'     => FALSE );

    defined $boolean or return undef;
    $boolean = lc $boolean;
    return $map{$boolean};
}


#
# Cette fonction prend un nombre entier en argument.  Si ce n'est pas
# un entier, la fonction retourne `undef' sinon elle retourne le nombre 
# lui-meme.  La fonction prend deux arguments optionnels designant
# respectivment le minimum et le maximum acceptes.  Si l'un d'entre eux est
# defini, le test est effectue et s'il echoue, la fonction retourne `undef'.
# Notons que $min et $max sont supposes corrects.
# ex:	Entree			Sortie
#	1			1
#	-1			undef
#	+1			1
#	-+1			undef
#	1, 0			1
#	1, 2			undef
#	10, 2, 20		10
#	10, 2, 8		undef
sub Is_Number($;$$) {
    my ($num, $min, $max) = @_;

    defined $num or return undef;
    $num =~ /^[+]?\d+$/ or return undef;
    # If $min or $max is defined, then return an error if the condition
    # is not true.
    defined $min and (not $num >= $min) and return undef;
    defined $max and (not $num <= $max) and return undef;

    # Supprime les eventuels "+" ou "0" inutiles.
    return sprintf "%d", $num;
}


#
# Cette fonction prend un nombre entier relatif en argument.  Si ce n'est pas
# un entier relatif, la fonction retourne `undef' sinon elle retourne le nombre 
# lui-meme.  La fonction prend deux arguments optionnels designant
# respectivment le minimum et le maximum acceptes.  Si l'un d'entre eux est
# defini, le test est effectue et s'il echoue, la fonction retourne `undef'.
# Notons que $min et $max sont supposes corrects.
# ex:	Entree			Sortie
#	1			1
#	-1			-1
#	+1			1
#	-+1			undef
#	1, 0			1
#	1, 2			undef
#	10, 2, 20		10
#	10, 2, 8		undef
sub Is_Integer($;$$) {
    my ($num, $min, $max) = @_;

    defined $num or return undef;
    $num =~ /^[-+]?\d+$/ or return undef;
    # If $min or $max is defined, then return an error if the condition
    # is not true.
    defined $min and (not $num >= $min) and return undef;
    defined $max and (not $num <= $max) and return undef;

    # Supprime les eventuels "+", "-" ou "0" inutiles.
    return sprintf "%d", $num;
}


#
# Cette fonction prend un numero de VLAN en argument.  Si ce n'est pas un
# numero de VLAN valide, la fonction retourne `undef' sinon elle retourne
# le numero de VLAN lui-meme.
# ex:	Entree			Sortie
#	1			1
#	4096			4096
#	0			undef
#	4097			undef
sub Is_Vlan($) {
    my ($vlan) = @_;

    defined $vlan or return undef;
    # VLAN 0 means no VLAN.
    # VLAN 1 is valid but used specially by Cisco, so you should avoid using it.
    return ($vlan =~ m/^\d+$/ && $vlan >= 1 && $vlan <= 4096) ? $vlan : undef;
}


#
# Cette fonction prend un VRID en argument.  Si ce n'est pas un VRID valide,
# la fonction retourne `undef' sinon elle retourne le VRID lui-meme.
# ex:	Entree			Sortie
#	0			0
#	255			255
#	256			undef
sub Is_Vrid($) {
    my $vrid = shift;   

    defined $vrid or return undef;
    return ($vrid =~ m/^\d+$/ && $vrid >= 0 && $vrid <= 255) ? $vrid : undef;
}   


#
# Cette fonction prend une adresse IPv4 en argument.  Si ce n'est pas une
# adresse IPv4 valide, la fonction retourne `undef' sinon elle retourne
# l'adresse IPv4 elle-meme.
# ex:	Entree			Sortie
#	"192.168.0.1"		"192.168.0.1"
#	"192.268.0.1"		undef
#	"192.168..0.1"		undef
sub Is_Ipv4($) {
	my $ip = shift;
	if ( Is_IPv4($ip) ) {
		$ip =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
		my ( $a, $b, $c, $d ) = ( $1, $2, $3, $4 );
		$a += 0;
		$b += 0;
		$c += 0;
		$d += 0;
		$ip = "$a.$b.$c.$d";
		return $ip;
	} else {
		return undef;
	}
}


#
# Cette fonction prend un port (TCP ou UDP) en argument.  Si ce n'est pas
# un port valide, la fonction retourne `undef' sinon elle retourne le port
# lui-meme.
# ex:	Entree			Sortie
#	1			1
#	0			undef
#	65536			undef
sub Is_Port($) {
    my $port = shift;

    defined $port or return undef;
    $port =~ m/^\d+$/ && $port >= 1 && $port <= 65535 or return undef;

    # Supprime les enventuels "0" inutiles.
    return $port+0;
}


#
# Cette fonction prend un interval de port en arugment (1-65535).  Si ce
# n'est pas un interval de ports valide, la fonction retourne `undef' sinon
# elle retourne l'interval de port lui-meme.  En pratique, la fonction
# utilise Is_Port() pour tester s'il s'agit d'un port.
# ex:	Entree			Sortie
#	"1-65535"		"0-65535"
#	"5000-1024"		undef
sub Is_Port_Range($) {
    my $portrange = shift;
    my @tmp;

    defined $portrange or return undef;
    @tmp = split '-', $portrange, -1;   # Do not remove trailing empty fields.
    @tmp == 2 or return undef;
    foreach (@tmp) {
	# Supprime les enventuels "0" inutiles.
	$_ = Is_Port($_) or return undef;
    }
    $tmp[0] < $tmp[1] or return undef;

    return $tmp[0].'-'.$tmp[1];
}

use constant CIDR_SHORT => 0;
use constant CIDR_LONG => 1;

# Cette fonction prend un CIDR en parametre.  Elle verifie son format et
# le reecrit selon le format desire si aucune erreur ne se produit.  Les
# formats acceptes sont :
#	o Une adresse IPv4 seule (on suppose un /32)
#	o Un subnet sous la forme xxx.xxx.xxx.xxx/n
#	o Un subnet sous la formet xxx.xxx.xxx.xxx/xxx.xxx.xxx.xxx
# Si le format est invalide, alors la fonction retourne `undef' sinon elle
# retourne le CIDR au format xxx.xxx.xxx.xxx/n par defaut.
# Un argument optionnel determine quel format de retour on souhaite :
#	o CIDR_SHORT: xxx.xxx.xxx.xxx/n
#	o CIDR_LONG: xxx.xxx.xxx.xxx/xxx.xxx.xxx.xxx
#
# ex:	Entree				Sortie
#	"192.168.0.1"			"192.168.0.1/32"
#	"192.168.0.1", VAL_CIDR_SHORT	"192.168.0.1/32"
#	"192.168.0.1", VAL_CIDR_LONG	"192.168.0.1/255.255.255.255"
#	"192.168.0.1/24", VAL_CIDR_LONG	"192.168.0.1/255.255.255.0"
#	"10.0.0.1/255.0.0.0"		"10.0.0.1/8"
#	"10.0.0.1/255.0.0"		undef
#	"10.0.0..1"			undef
sub Is_Cidr($;$) {
    my ($cidr, $query) = @_;
    my (@tmp, $ip, $netmask);
    my ($short, $long);
    my $bitfield;

    defined $cidr or return undef;
    defined $query or $query = CIDR_SHORT;

    @tmp = split '/', $cidr, -1;        # Do not remove trailing empty fields.
    @tmp == 1 and push @tmp, '32';      # For IP only, fake a "/32" suffix.
    # There must be two parts : IP address and netmask.
    @tmp == 2 or return undef;
    ($ip, $netmask) = @tmp;
    # Supprime les eventuels "0" inutiles.
    $ip = Is_Ipv4($ip) or return undef;

    SWITCH: {
        ($netmask =~ m/^\d+$/) and do { # Netmask in decimal notation.
            $netmask > 0 && $netmask <= 32 or return undef;

	    # Supprime les eventuel "0" inutiles.
	    $netmask = sprintf "%d", $netmask;

            $short = $ip.'/'.$netmask;

            $bitfield = '1' x $netmask . '0' x (32 - $netmask);
            $long = $ip.'/'.join '.', unpack 'C4', pack 'B32', $bitfield;
            last SWITCH;
        };

        Is_Netmask($netmask) and do {       # Netmask in dotted notation.
            my $obit = 1;
            my $nbit;

            # Convert to a stringwise bitfield.
            $bitfield = unpack 'B32', pack 'C4', split '\.', $netmask;
            # Checks if any '1' after a '0'.
            for (my $i = 0; $i < length $bitfield; $i++) {
                $nbit = substr $bitfield, $i, 1;

                # This should not occur, this mean the netmask is invalid.
                $obit == 0 && $nbit == 1 and return undef;
                # End of the network part mask.
                $obit == 1 && $nbit == 0 and $short = $ip.'/'.$i;
                $obit = $nbit;
            }
            # This is a /32 netmask.
            defined $short or $short = $ip.'/32';
            $long = $ip.'/'.$netmask;
            last SWITCH;
        };

        return undef;
    }

    $cidr = $query == CIDR_LONG ? $long : $short;
    return $cidr;
}


# Cette fonction prend un hote en argument (un hote est soit un nom DNS,
# soit une adresse IP).  Si ce n'est pas l'un des deux formats, la fonction
# retourne `undef' sinon elle retourne l'hote lui-meme.  En pratique elle
# verifie s'il s'agit d'un domaine avec Is_Domain() ou d'une adresse IPv4
# avec Val_Is_Ipv4().
sub Is_Host($) {
    my $host = shift;

    defined $host or return undef;
    Is_RFC822Domain($host) and return $host;

    $host = Is_Ipv4($host);
    defined $host and return $host;

    return undef;
}


# Cette fonction prend un OID (Object Identifier dans une MIB SNMP).  Si
# ce n'est pas un OID valide, la fonction retourne `undef', sinon elle
# retourne l'OID lui-meme dont on aura supprime l'eventuel prefix ".".
# ex:	Entree		Sortie
#	.1.3.6.1.4.1	1.3.6.1.4.1
#	1.3.6.1.4.1	1.3.6.1.4.1
#	1.3.6.1.4.	undef
#	1.3.6.1.4..1	undef
sub Is_Mib_Oid($) {
    my ($oid) = @_;
    my @oid;
    my ($i, $tmp);

    defined $oid or return undef;
    @oid = split /\./, $oid, -1;
    @oid >= 2 or return undef;
    # Allow an OID beginning with a dot.
    if ($oid[0] eq '') {
	@oid == 1 and return undef;
	shift @oid;
    }

    # Check the OIDs up to the last but one.
    for ($i = 0; $i < (@oid - 1); $i++) {
	# Supprimes les eventuels "0" inutiles.
	$tmp = Is_Number($oid[$i], 1);
	defined $tmp or return undef;
	$oid[$i] = $tmp;
    }

    # Here $i is scalar (@oid) - 1.
    # The last part may be 0 (scalar node not belonging to an array).
    $tmp = Is_Number($oid[$i], 0);
    defined $tmp or return undef;
    $oid[$i] = $tmp;

    return join '.', @oid;
}


# Cette fonction renvoie l'index de l'element s'il se trouve dans le
# tableau (comparaison "stringwise" a l'aide l'operateur `eq') sinon la
# fonction retourne -1.
sub Is_Member($$) {
    my ($item, $array) = @_;

    defined $item or return undef;
    defined $array or return undef;
    ref ($array) eq 'ARRAY' or return undef;
    for (my $i = 0; $i < @$array; $i++) {
	$item eq $array->[$i] and return $i;
    }
    return -1;
}

1 ;
